#Region " Autodesk Copyright"
'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region "Namespaces References"
Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports PartType = Autodesk.Aec.Building.DatabaseServices.Type
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AddToCatalog.AddToCatalog))> 

Namespace AddToCatalog
    Public Class AddToCatalog

#Region "Member Data"

        Private mEditor As Editor
        Private mDb As Database
        Private mDoc As Document
        Private mTransMgr As TransactionManager
        Private mCatalog As CatalogManager

#End Region

#Region "Helper Setup"

        Protected Sub Setup()
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mDoc = AcadApplication.DocumentManager.MdiActiveDocument
            mTransMgr = mDb.TransactionManager
            mCatalog = New CatalogManager
        End Sub

#End Region

#Region "Command AddCustomData"

        <Autodesk.AutoCAD.Runtime.CommandMethod("CatalogApiSample", "AddCustomData", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub AddCustomData()
            Try
                Setup()
                ' We need to create a part lookup object prior to loading the raw table of the part.  
                ' We will query the duct catalog for a round duct.  The catalog being queried
                ' can be specified through the options dialog.

                Dim query As DataQuery = New DataQuery

                query.Domain = Domain.DuctComponent 'Set the domain on the query
                query.Type = PartType.Duct ' Set the part type - in this case a duct
                query.SetPortShape(1, BuiltInShape.Circular) ' Set the port shape

                Dim partslist As DataPartsList = mCatalog.GetPartsList(query)

                If partslist.DataParts.Count > 0 Then ' Check that we we got at least 1 result - we may have more

                    Dim dpl As DataPartLookup
                    dpl = partslist.DataParts(0)

                    If dpl <> Nothing Then

                        ' Now, that we have the part lookup object, we can get the raw table of the part.  The raw table
                        ' represents a run time version of the xml data for the part.
                        '
                        Dim rawTable As DataRawTable = mCatalog.GetRawPartTable(dpl)

                        ' get the collection of ColumnConsts from the raw table
                        '
                        Dim dfc As DataFieldCollection = rawTable.ConstSet

                        ' Get the next available index from the raw table.
                        '
                        Dim index As Integer
                        index = rawTable.GetNextFreeParameterIndex(Context.CustomData)

                        ' Create our new DataField that we will add to the raw table
                        '
                        Dim field As DataField = New DataField(Context.CustomData, index)

                        ' Set the properties on the DataField.  In this sample, we will set the name, description
                        ' and an integer value.
                        '
                        Dim sIndex As String = field.Index.ToString

                        field.Name = "CD" + sIndex

                        field.Description = "CustomData" + sIndex

                        field.ValueInteger = 100

                        ' Now add the data field created to the raw table.
                        '
                        rawTable.ConstSet.Add(field)

                        ' Save the raw table to write the changes we have made back to the xml file and update the Catalog  
                        '
                        mCatalog.AddRawPartTable(dpl.Domain, dpl, rawTable, True)

                    End If
                End If

            Catch ex As System.Exception

                Debug.Write(String.Format("" & Microsoft.VisualBasic.Chr(10) & "Caught the following Exception:{0}", ex.ToString))

            End Try

        End Sub

#End Region


    End Class
End Namespace

