#Region "Header"
'      .NET Sample
'
'      Copyright (c) 2006 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Modeler
Imports Autodesk.AutoCAD.EditorInput


Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(DisplaySample))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(DisplaySample))> 

Public Class DisplaySample
    Implements IExtensionApplication

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading Display System Sample..." & vbCrLf)
        ed.WriteMessage("Input ""DisplaySample"" to run the sample..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "Command"
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecDisplaySampleMgd", "DisplaySample", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub DisplayOverrides()
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("====================================================" & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Display System Sample:" & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("====================================================" & vbCrLf)

        DisplayConfigs()

        ListDisplaySets()

        DisplayReps()

        DisplayProps()

        DisplayPropsMaterial()

    End Sub
#End Region

    Private m_i As Integer

    Sub DisplayConfigs()

        '   access the display system from the database object

        Dim db As Autodesk.AutoCAD.DatabaseServices.Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim trans As Autodesk.AutoCAD.DatabaseServices.Transaction = db.TransactionManager.StartTransaction()

        Try

            '   access the display system from the document object
            Dim dictDisplayConfigs As New DictionaryDisplayConfiguration(db)

            '  the total number of display configurations registered. 
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & dictDisplayConfigs.Records.Count & " display configs" & vbCrLf)

            '  list them
            Dim dcId As ObjectId
            For Each dcId In dictDisplayConfigs.Records
                Dim dc As DisplayConfiguration = trans.GetObject(dcId, OpenMode.ForWrite)
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Name : " & dc.Name & vbCrLf & vbCrLf & "Description :  " & dc.Description & vbCrLf)
            Next dcId

            '  add a new config with the name "MyConfig" with a description in it. 
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Let's add a New DisplayConfiguration:" & vbCrLf)
            Dim displayconfig As New DisplayConfiguration
            displayconfig.Description = "This is MyConfig"
            If dictDisplayConfigs.Has("MyConfig", trans) <> True Then
                dictDisplayConfigs.AddNewRecord("MyConfig", displayconfig)
                trans.AddNewlyCreatedDBObject(displayconfig, True)
            End If

            '  add another one "FORUSE1"
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(displayconfig.Name & " : " & displayconfig.Description & vbCrLf)
            Dim displayconfig1 As New DisplayConfiguration
            displayconfig1.Description = "This is for use"
            If dictDisplayConfigs.Has("FORUSE1", trans) <> True Then
                dictDisplayConfigs.AddNewRecord("FORUSE1", displayconfig1)
                trans.AddNewlyCreatedDBObject(displayconfig1, True)
            End If

            '  rename MyConfig to a new name MyNewConfigureXxx 
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Let's rename this New DisplayConfiguration:" & vbCrLf)
            Dim newName As String = GetNewName()
            dictDisplayConfigs.Rename("MyConfig", newName, trans)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("After Renamed: " & vbCrLf & "Name: " & displayconfig.Name & vbCrLf & "Description: " & displayconfig.Description & vbCrLf)


            '  report the total number of display config again. 
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("There are " & dictDisplayConfigs.Records.Count & " display configurations now." & vbCrLf)

            Dim id As ObjectId = dictDisplayConfigs.GetAt(newName)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("The current ObjectId for New display configuration is:" & id.ToString() & vbCrLf)

            Dim drm As New DisplayRepresentationManager(db)
            Dim currentDisConfId As ObjectId = drm.DisplayConfigurationIdForCurrentViewport
            Dim currentDisConf As DisplayConfiguration = trans.GetObject(currentDisConfId, OpenMode.ForRead)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("The Current DisplayConfiguration is " & currentDisConf.Name & vbCrLf)
            ListDisplayConfig(currentDisConf)

            '  create a new disp config with more detailed information. 
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Let's create a New DisplayConfiguration for detail:" & vbCrLf)
            Dim newDc As DisplayConfiguration = CreateDisplayConfig()

            '  print it out. 
            ListDisplayConfig(newDc)

            trans.Commit()

        Catch e As System.Exception
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Exception: " & e.Message & vbCrLf)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub

    '  create a display config with a little more information filled. 
    '
    Private Function CreateDisplayConfig() As DisplayConfiguration
        Dim db As Autodesk.AutoCAD.DatabaseServices.Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim trans As Transaction = db.TransactionManager.StartTransaction()

        Dim drm As New DisplayRepresentationManager(db)
        Dim currentDisConfId As ObjectId = drm.DisplayConfigurationIdForCurrentViewport
        Dim currentDisConf As DisplayConfiguration = trans.GetObject(currentDisConfId, OpenMode.ForRead)

        Dim dc As New DisplayConfiguration()

        Try

            Dim ds As New DisplaySet()
            ds.SetToStandard(db)
            ds.SubSetDatabaseDefaults(db)

            dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(0, 0, 1), DictionaryDisplaySet.GetStandardPlanId(db))) ' top

            Dim id As ObjectId = DictionaryDisplaySet.GetStandardSet("Plan Diagnostic", db)
            If Not (id = Nothing) Then
                dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(0, 0, -1), id)) ' bottom 
            Else ' if not found, let's use Reflected. 
                dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(0, 0, -1), DictionaryDisplaySet.GetStandardReflectedId(db))) ' bottom 
            End If

            dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(-1, 0, 0), DictionaryDisplaySet.GetStandardSectionElevId(db))) ' left
            dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(1, 0, 0), DictionaryDisplaySet.GetStandardSectionElevId(db))) ' right
            dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(0, -1, 0), DictionaryDisplaySet.GetStandardSectionElevId(db))) ' front
            dc.ViewDependentCombinations.Add(New ViewDependentCombination(New Vector3d(0, 1, 0), DictionaryDisplaySet.GetStandardSectionElevId(db)))  ' back

            dc.DefaultViewDependentViewSet = DictionaryDisplaySet.GetStandardModelId(db)

            Dim dictDisplayConfigs As New DictionaryDisplayConfiguration(db)
            Dim newName As String = GetNewName()
            dictDisplayConfigs.AddNewRecord(newName, dc)
            trans.AddNewlyCreatedDBObject(dc, True)
            trans.Commit()

        Catch e As System.Exception
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

        Return dc

    End Function

    '  show the information of the given display configuration. 
    '
    Private Sub ListDisplayConfig(ByVal dc As DisplayConfiguration)
        Dim db As Autodesk.AutoCAD.DatabaseServices.Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim trans As Transaction = db.TransactionManager.StartTransaction()

        Try
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Name: " & dc.Name & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Description: " & dc.Description & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Cut Plane Display Above: " & dc.CutPlaneAboveRange & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Cut Plane Display Below: " & dc.CutPlaneBelowRange & vbCrLf)

            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Use View Port View Direction: " & dc.UseViewportViewDirection.ToString() & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Is View Dependent: " & dc.IsViewDependent.ToString() & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Hard View Direction: " & dc.HardViewDirection.ToString() & vbCrLf)


            Dim v As Vector3d
            Dim ds As DisplaySet
            Dim vd As ViewDependentCombination
            For Each vd In dc.ViewDependentCombinations
                v = vd.Direction
                ds = trans.GetObject(vd.SetId, OpenMode.ForRead)
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(v.ToString() & " " & ds.Name & vbCrLf)
            Next vd

            ds = trans.GetObject(dc.DefaultViewDependentViewSet, OpenMode.ForRead)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Default View Dependent View Set: " & ds.Name & vbCrLf)

            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)
            trans.Commit()

        Catch e As System.Exception
            trans.Abort()
        Finally
            trans.Dispose()
        End Try


    End Sub

    '  list all the display reps for MassElem and MassGroup for the current display config. 
    '

    Private Sub DisplayReps()

        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("====================================================" & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Get All the AecDbMassElem/Massgroup Display Reps for the current display config" & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("====================================================" & vbCrLf)

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim trans As Transaction = db.TransactionManager.StartTransaction()
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim ids As Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection = DisplayRepresentationManager.GetActiveDisplayRepresentationSets(db)
        ed.WriteMessage("# of active display sets = " & ids.Count & vbCrLf)

        'Dim acds As DisplaySet 
        Dim ds As DisplaySet = New DisplaySet()

        Try
            Dim id As ObjectId
            ' loop through the active disp sets (i.e., disp sets of active dis confid.) 
            For Each id In ids
                Dim acds As DisplaySet = trans.GetObject(id, OpenMode.ForRead)

                Dim drid As ObjectId
                ' look through the set of dispRep of the given disp set. 
                For Each drid In acds.DisplayRepresentationIds
                    Dim dr As DisplayRepresentation = trans.GetObject(drid, OpenMode.ForRead)
                    dr = trans.GetObject(drid, OpenMode.ForRead)
                    Dim rc As RXClass = dr.WorksWith
                    If (rc.Name.CompareTo("AecDbMassElem") = 0 Or rc.Name.CompareTo("AecDbMassGroup") = 0) Then
                        If (ds.DisplayRepresentationIds.Contains(drid)) Then
                            Continue For
                        Else
                            ds.DisplayRepresentationIds.Add(drid)
                            ListDisplayRep(dr)
                        End If
                    End If
                Next drid
            Next id
            trans.Commit()

        Catch e As System.Exception
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub

    '   not used in the original code, so added ListDisplaySets below. 
    ' 
    Private Sub ListDisplaySet(ByVal ds As DisplaySet)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplaySet Name:" & ds.Name & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplaySet Description:" & ds.Description & vbCrLf)
    End Sub

    '   added. list all the disp set registered. 
    '
    Private Sub ListDisplaySets()

        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("===================================" & vbCrLf)
        ed.WriteMessage("  Currently Registerd Display Set  " & vbCrLf)
        ed.WriteMessage("===================================" & vbCrLf)

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim trans As Transaction = db.TransactionManager.StartTransaction()

        ' get the dictionary. 
        Dim dictDispSet As DictionaryDisplaySet = New DictionaryDisplaySet(db, False)
        ed.WriteMessage("Current # of display set = " & dictDispSet.Records.Count & vbCrLf)

        Try
            Dim id As ObjectId
            ' loop throught it and print out the name. 
            For Each id In dictDispSet.Records
                Dim dispSet As DisplaySet = trans.GetObject(id, OpenMode.ForRead)
                ListDisplaySet(dispSet)
            Next id

        Catch ex As Exception
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub

    Private Sub ListDisplayRep(ByVal dr As DisplayRepresentation)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplayRep Name: " & dr.Name & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplayRep Work With: " & dr.WorksWith.Name & vbCrLf)

        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplayRep Display Name: " & dr.DisplayName & vbCrLf)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("DisplayRep Display Representation Name: " & dr.DisplayRepresentationName & vbCrLf)

    End Sub

    '  list all the disp reps for the door for the current view port 
    '
    Private Sub DisplayProps()
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim trans As Transaction = db.TransactionManager.StartTransaction()
        Dim bt As BlockTable = tm.GetObject(db.BlockTableId, OpenMode.ForRead, False)
        Dim btr As BlockTableRecord = tm.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False)
        Dim dds As DictionaryDisplaySet = New DictionaryDisplaySet(db, False)

        Try

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            Dim optEnt As New PromptEntityOptions(vbLf & "Select an AEC door entity")
            optEnt.SetRejectMessage(vbLf & "Selected entity is NOT an AEC door entity, try again...")
            optEnt.AddAllowedClass(GetType(Autodesk.Aec.Arch.DatabaseServices.Door), False)  ' Geo is the base class of AEC entities. 

            Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
            If resEnt.Status <> PromptStatus.OK Then
                ed.WriteMessage("Selection error - aborting")
                Exit Sub
            End If

            Dim door As Autodesk.Aec.Arch.DatabaseServices.Door = trans.GetObject(resEnt.ObjectId, OpenMode.ForRead)
            ed.WriteMessage("You have selected an " & door.GetRXClass.Name & vbCrLf)

            ' get its style
            Dim doorstyle As Autodesk.Aec.Arch.DatabaseServices.DoorStyle = trans.GetObject(door.StyleId, OpenMode.ForRead)


            Dim doorId As ObjectId = door.ObjectId
            Dim drm As DisplayRepresentationManager = New DisplayRepresentationManager(db)

            Dim ids As Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection = drm.GetDisplayRepresentationIdsFromCurrentViewport(door.GetRXClass())
            Dim dr As DisplayRepresentation
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Get all the Display Representations work with the door for current view port" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("door disp rep count = " & ids.Count & vbCrLf)

            ' 
            For Each id As ObjectId In ids
                dr = trans.GetObject(id, OpenMode.ForRead)

                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(vbCrLf & "Display Representation Name: " & dr.Name & " Display Name: " & dr.DisplayName & vbCrLf)

                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Display Representation Name: " & dr.Name & "Display Name" & dr.DisplayName & vbCrLf)
                Dim dps As DisplayProperties
                Dim rcProps As RXClass = dr.DisplayPropertiesClass
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Display Props Class: " & rcProps.Name & vbCrLf)

                ' find override at instance level. 
                Dim overId As ObjectId = dr.FindDisplayPropertiesOverride(door)
                If (overId <> ObjectId.Null) Then
                    dps = trans.GetObject(overId, OpenMode.ForRead)
                    Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Element Override Display Props Name: " & dps.Name & " Display Name:" & dps.DisplayName & vbCrLf)
                    Continue For
                End If

                Dim overstyleId As ObjectId = dr.FindDisplayPropertiesOverride(doorstyle)
                If (overstyleId <> ObjectId.Null) Then
                    dps = trans.GetObject(overstyleId, OpenMode.ForRead)
                    Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Style Override Display Props Name: " & dps.Name & " Display Name:" & dps.DisplayName & vbCrLf)
                    Continue For
                End If

                Dim defaultDpsId As ObjectId = dr.DefaultDisplayPropertiesId
                If Not defaultDpsId.IsNull Then
                    dps = trans.GetObject(defaultDpsId, OpenMode.ForRead)
                    Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Default Display Props Name: " & dps.Name & " Display Name:" & dps.DisplayName & vbCrLf)
                End If

            Next id
            trans.Commit()

        Catch
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub

    Private Sub DisplayPropsMaterial()
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
        Dim trans As Transaction = db.TransactionManager.StartTransaction()

        Try
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Get all the Display Properties and the Display components works with AecDbMaterialDef in ZAxis view" & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("=======================================" & vbCrLf)

            Dim drm As DisplayRepresentationManager = New DisplayRepresentationManager(db)
            Dim vport As Viewport = New Viewport()
            Dim btr As BlockTableRecord = trans.GetObject(db.CurrentSpaceId, OpenMode.ForWrite, False)
            btr.AppendEntity(vport)
            trans.AddNewlyCreatedDBObject(vport, True)
            Dim viewportId As ObjectId = vport.ObjectId

            Dim viewDirection As Vector3d = Vector3d.ZAxis
            Dim setId As ObjectId = drm.DisplayRepresentationSet(viewportId, viewDirection)
            Dim ds As DisplaySet = trans.GetObject(setId, OpenMode.ForWrite)
            Dim repIds As DisplayRepresentationIdCollection = ds.DisplayRepresentationIds
            Dim dr As DisplayRepresentation
            For Each repId As ObjectId In repIds
                dr = trans.GetObject(repId, OpenMode.ForRead)
                If (dr.WorksWith.Name = "AecDbMaterialDef") Then
                    Dim dpId As ObjectId = dr.DefaultDisplayPropertiesId
                    Dim dp As DisplayProperties = trans.GetObject(dpId, OpenMode.ForWrite)
                    ListDisplayProperties(dp)
                End If
            Next repId
        Catch e As System.Exception
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(e.Message & vbCrLf)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
        Return
    End Sub

    Private Sub ListDisplayProperties(ByVal dp As DisplayProperties)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("List Display Properties: Name:" & dp.Name & vbCrLf)


        Dim sc As System.Collections.Specialized.StringCollection = New System.Collections.Specialized.StringCollection()

        Dim components() As DisplayComponent = dp.GetDisplayComponents(sc)
        If components.Length = 0 Then
            Return
        End If
        Dim i As Integer
        For i = 0 To components.Length - 1


            Dim dc As DisplayComponent = components(i)
            Dim name As String = sc(i)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("List Display Components: " & name & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Is Applicable? : " & dc.IsApplicable.ToString() & vbCrLf)
            Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("Is Inherited?: " & dc.IsInherited.ToString() & vbCrLf)

            If (TypeOf (dc) Is DisplayComponentEntity) Then
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("It is a Display Component Entity. " & vbCrLf)
            End If
            If (TypeOf (dc) Is DisplayComponentHatch) Then
                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("It is a Display Component Hatch." & vbCrLf)
            End If
        Next i
    End Sub

    Private Function GetNewName() As String
        Dim newString As String = "MyNewConfigure" & m_i.ToString()
        m_i = m_i + 1
        Return newString
    End Function


End Class












