#Region "Copyright"
'      .NET ProjectBase Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Namespaces"
Imports System
Imports System.Collections.Specialized
Imports System.Text
Imports System.IO

Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Project

Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(VFS))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(VFS))> 

Public Class VFS
    Implements IExtensionApplication

#Region "Data Members"
    Private Shared mgr As ProjectBaseManager = ProjectBaseServices.Service.ProjectManager
    Private Shared testproject As String = "C:\temp\Sample Project\"
    Private Shared name As String = "Sample Project"
    Private Shared proj As Project = Nothing
    Private Shared ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
    Private Shared topOpts As PromptKeywordOptions = New PromptKeywordOptions("Commands")
    Private Shared currentPath As String = testproject

#End Region

#Region "IExtensionApplication"

    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Print("Initializing Sample Project...")

        Dim sourceDir As String = Environment.GetEnvironmentVariable("USERPROFILE")
        sourceDir += "\My Documents\Autodesk\My Projects\Sample Project 2009\"

        If (Not Directory.Exists(testproject)) Then
            CopyDir(sourceDir, testproject)
        End If

        If (proj = Nothing) Then
            proj = mgr.OpenProject(OpenMode.ForRead, testproject, name)
        End If

        ' top commands
        topOpts.Keywords.Add("ls")
        topOpts.Keywords.Add("cd")
        topOpts.Keywords.Add("open")
        topOpts.Keywords.Add("pwd")
        topOpts.Keywords.Add("check")
        topOpts.Keywords.Add("mode")
        topOpts.Keywords.Add("upgrade")
        topOpts.Keywords.Add("downgrade")
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub

#End Region

#Region "Print"
    Private Sub Print(ByVal msg As String)
        Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(msg + vbCrLf)
    End Sub
#End Region

#Region "CopyDir"
    Private Sub CopyDir(ByVal src As String, ByVal dest As String)
        If (dest(dest.Length - 1) <> Path.DirectorySeparatorChar) Then
            dest += Path.DirectorySeparatorChar
        End If

        If (Not Directory.Exists(dest)) Then
            Directory.CreateDirectory(dest)
        End If

        Dim fileList As String() = Directory.GetFileSystemEntries(src)

        Dim fileName As String
        For Each fileName In fileList
            ' directory, recursively copy
            If (Directory.Exists(fileName)) Then
                CopyDir(fileName, dest + Path.GetFileName(fileName))
            Else ' file
                File.Copy(fileName, dest + Path.GetFileName(fileName), True)
                Dim attri As FileAttribute = File.GetAttributes(fileName)
                attri = attri And (Not FileAttributes.ReadOnly)
                File.SetAttributes(dest + Path.GetFileName(fileName), attri)
            End If
        Next

    End Sub
#End Region

#Region "Command Methods"
    <Autodesk.AutoCAD.Runtime.CommandMethod("AecProjectBaseSampleMgd", "ProjectSample", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub Project()
        While (True)
            Dim result As PromptResult = ed.GetKeywords(topOpts)
            If (result.Status = PromptStatus.OK And result.StringResult <> "exit") Then
                Dim cmd As String = result.StringResult
                Handler(cmd)
            Else
                Exit While
            End If
        End While
    End Sub
#End Region

#Region "Help Functions"
    Private Sub ListAll()

        Dim fileList As String() = Directory.GetFileSystemEntries(currentPath)
        Dim category As FileCategory = proj.GetCategory(currentPath)
        Dim isCategory As Boolean = (category <> FileCategory.Unknown)

        Print("1. <CATEGORY>       .")
        Print("2. <CATEGORY>       ..")

        Dim i As Integer = 2
        Dim file As String
        For Each file In fileList
            ' find category or file in category 
            Dim belongToCategory As FileCategory = proj.GetFileCategory(file)
            Dim isViewed As Boolean = True
            Dim tag As String = Nothing

            ' in category directory
            If (belongToCategory <> FileCategory.Unknown) Then
                If (Not Directory.Exists(file)) Then
                    tag = ". <DWGFILE>        "
                    Dim projFileName As ProjectFileName = New ProjectFileName(file)
                    ' non-xml file
                    If (projFileName.Extension.ToLower() <> ".xml") Then
                        isViewed = False
                    Else ' xml file
                        Dim projFile As ProjectFile = proj.GetFile(file)
                        If (projFile = Nothing) Then
                            isViewed = False
                        ElseIf (Not projFile.DwgExists) Then
                            isViewed = False
                        End If
                    End If
                Else
                    tag = ". <CATEGORY>       "
                End If
            Else
                isViewed = False
            End If

            If (isViewed) Then
                i = i + 1
                Dim fileName As String = file.Substring(currentPath.Length)

                If (fileName.ToLower().Contains(".xml")) Then
                    fileName = fileName.Substring(0, fileName.Length - 4)
                End If
                Print(i.ToString() + tag + fileName)
            End If
        Next
    End Sub

    Private Function IsValidFolder(ByVal name As String) As Boolean
        Dim newPath As String = currentPath + name

        If (newPath(newPath.Length - 1) <> Path.DirectorySeparatorChar) Then
            newPath += Path.DirectorySeparatorChar
        End If

        Return (Directory.Exists(newPath))
    End Function

    Private Sub Update(ByVal name As String)
        If (currentPath(currentPath.Length - 1) <> Path.DirectorySeparatorChar) Then
            currentPath += Path.DirectorySeparatorChar
        End If

        currentPath += name

        If (currentPath(currentPath.Length - 1) <> Path.DirectorySeparatorChar) Then
            currentPath += Path.DirectorySeparatorChar
        End If
    End Sub

    Private Function GetFullPath(ByVal index As Integer) As String
        Dim returnPath As String = currentPath

        ' parent path
        If (index = 2) Then
            If (returnPath <> testproject) Then
                returnPath = returnPath.Remove(returnPath.Length - 1)
                Dim pos As Integer = returnPath.LastIndexOf(Path.DirectorySeparatorChar)
                returnPath = returnPath.Substring(0, pos + 1)
            End If
            ' specified path
        ElseIf (index > 2) Then
            Dim fileList As String() = Directory.GetFileSystemEntries(returnPath)
            Dim i As Integer = 2
            Dim file As String

            For Each file In fileList
                Dim belongToCategory As FileCategory = proj.GetFileCategory(file)
                Dim isViewed As Boolean = True
                If (belongToCategory <> FileCategory.Unknown) Then
                    If (Not Directory.Exists(file)) Then
                        Dim projFileName As ProjectFileName = New ProjectFileName(file)
                        ' non-xml file
                        If (projFileName.Extension.ToLower() <> ".xml") Then
                            isViewed = False
                        Else ' xml file
                            Dim projFile As ProjectFile = proj.GetFile(file)
                            If (projFile = Nothing) Then
                                isViewed = False
                            ElseIf (Not projFile.DwgExists) Then
                                isViewed = False
                            End If
                        End If
                    End If
                Else
                    isViewed = False
                End If

                If (isViewed) Then
                    i = i + 1
                End If

                If (i = index) Then
                    returnPath = file
                    Exit For
                End If
            Next
        End If
        ' Else: current path

        If (Directory.Exists(returnPath) And returnPath(returnPath.Length - 1) <> Path.DirectorySeparatorChar) Then
            returnPath += Path.DirectorySeparatorChar
        End If

        Return returnPath
    End Function

    Private Sub Open(ByVal path As String)
        Dim projFile As ProjectFile = proj.GetFile(path)

        If (projFile.DwgExists) Then
            Application.DocumentManager.Open(projFile.DrawingFullPath, True)
        Else
            Print(path + " does not exist!")
        End If
    End Sub

    Private Function IsSamePath(ByVal path1 As String, ByVal path2 As String) As Boolean
        Return (path1.ToLower() = path2.ToLower())
    End Function

    Private Function GetDbForFile(ByVal dwgFullPath As String) As Database
        Dim docs As DocumentCollection = Application.DocumentManager
        Dim doc As Document = Nothing

        Dim elem As Document
        For Each elem In docs
            If (IsSamePath(elem.Database.Filename, dwgFullPath)) Then
                doc = elem
                Exit For
            End If
        Next

        If (doc <> Nothing) Then
            Return doc.Database
        Else
            Dim db As Database = New Database(False, True)
            db.ReadDwgFile(dwgFullPath, FileShare.Read, False, Nothing)
            db.ResolveXrefs(False, True)
            Return db
        End If
    End Function

    Private Sub Check(ByVal path As String)
        Dim file As ProjectFile = proj.GetFile(path)

        If (File.DwgExists) Then
            ' Gets the database for the specified dwg file
            Dim dwgName As String = file.DrawingFullPath

            Dim dwgDb As Database = GetDbForFile(dwgName)

            If (dwgDb <> Nothing) Then
                Dim symTbId As ObjectId = dwgDb.BlockTableId
                Dim trans As Transaction = dwgDb.TransactionManager.StartTransaction()

                Try
                    Dim bt As BlockTable = trans.GetObject(symTbId, OpenMode.ForRead, False)
                    Dim refs As StringCollection = New StringCollection()

                    Dim recId As ObjectId
                    For Each recId In bt
                        Dim btr As BlockTableRecord = trans.GetObject(recId, OpenMode.ForRead)

                        If (btr.IsFromExternalReference) Then
                            refs.Add(btr.Name)
                        End If
                    next

                    ' output referenced files
                    If (refs.Count = 0) Then
                        Print("<" + file.FileName + "> references no other dwg files.")
                    Else
                        Print("<" + file.FileName + "> references the following dwg files:")
                        Dim i As Integer = 0

                        Dim refName As String
                        For Each refName In refs
                            i = i + 1
                            Print(i.ToString() + ". " + refName)
                        Next
                    End If
                Catch
                    trans.Abort()
                Finally
                    trans.Dispose()
                End Try
            End If
        End If
    End Sub
#End Region

#Region "Handler"
    Private Sub Handler(ByVal cmd As String)
        Select Case cmd

            Case "ls"
                ListAll()

            Case "cd"
                Dim iOpt As PromptIntegerOptions = New PromptIntegerOptions("Input Folder Number")
                Dim result As PromptIntegerResult = ed.GetInteger(iOpt)
                If (result.Status = PromptStatus.OK) Then
                    Dim chosenPath As String = GetFullPath(result.Value)
                    If (Directory.Exists(chosenPath)) Then
                        currentPath = chosenPath
                        Exit Select
                    End If
                End If
                Print("You have chosen an invalid folder")

            Case "open"
                Dim iOpt2 As PromptIntegerOptions = New PromptIntegerOptions("Input File Number")
                Dim result2 As PromptIntegerResult = ed.GetInteger(iOpt2)
                If (result2.Status = PromptStatus.OK) Then
                    Dim chosenPath As String = GetFullPath(result2.Value)
                    If (Not Directory.Exists(chosenPath)) Then
                        Open(chosenPath)
                        Exit Select
                    End If
                End If
                Print("You have chosen an invalid file")

            Case "check"
                Dim iOpt3 As PromptIntegerOptions = New PromptIntegerOptions("Input File Number")
                Dim result3 As PromptIntegerResult = ed.GetInteger(iOpt3)
                If (result3.Status = PromptStatus.OK) Then
                    Dim chosenPath As String = GetFullPath(result3.Value)
                    If (Not Directory.Exists(chosenPath)) Then
                        Check(chosenPath)
                        Exit Select
                    End If
                End If
                Print("You have chosen an invalid file")

            Case "pwd"
                Dim wd As String = "\Sample\" + currentPath.Substring(testproject.Length)
                Print(wd)

            Case "mode"
                If (mgr.IsProjectLockedForWrite(testproject + name + ".apj")) Then
                    Print("This project is opened in write mode.")
                Else
                    Print("This project is opened in read-only mode.")
                End If

            Case "upgrade"
                If (Not mgr.IsProjectLockedForWrite(testproject + name + ".apj")) Then
                    mgr.UpgradeOpen(proj)
                    Print("Upgraded open succesfully.")
                Else
                    Print("This project is already opened in write mode.")
                End If

            Case "downgrade"
                If (mgr.IsProjectLockedForWrite(testproject + name + ".apj")) Then
                    mgr.DowngradeOpen(proj)
                    Print("Downgrade open succesfully.")
                Else
                    Print("This project is already opened in read mode.")
                End If
        End Select

    End Sub
#End Region

End Class




