#Region "Copyright"
'      .NET Stream Sample
'
'      Copyright (c) 2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Namespaces"
Imports System

Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices

Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
Imports DBObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports Database = Autodesk.AutoCAD.DatabaseServices.Database
Imports TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports Transaction = Autodesk.AutoCAD.DatabaseServices.Transaction
Imports OpenMode = Autodesk.AutoCAD.DatabaseServices.OpenMode
Imports LineWeight = Autodesk.AutoCAD.DatabaseServices.LineWeight
Imports BlockReference = Autodesk.AutoCAD.DatabaseServices.BlockReference
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecModeler = Autodesk.Aec.Modeler
Imports Autodesk.Aec.Geometry

Imports Microsoft.VisualBasic
#End Region

<Assembly: ExtensionApplication(GetType(AecStreamSampleMgd.StreamSample))> 
<Assembly: CommandClass(GetType(AecStreamSampleMgd.StreamSample))> 

Namespace AecStreamSampleMgd
    Public Class StreamSample
        Implements IExtensionApplication

#Region "IExtensionApplication"
        ''' <summary>
        ''' Initialization.
        ''' </summary>
        Public Sub Initialize() Implements IExtensionApplication.Initialize
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ed.WriteMessage("Loading Stream Sample..." + ControlChars.Cr + ControlChars.Lf)
            ed.WriteMessage("Type ""StreamSample"" to run the sample:" + ControlChars.Cr + ControlChars.Lf)
        End Sub

        ''' <summary>
        ''' Termination.
        ''' </summary>
        Public Sub Terminate() Implements IExtensionApplication.Terminate
        End Sub
#End Region

#Region "CommandMethod"
        ''' <summary>
        ''' Command implementation.
        ''' </summary>
        <Autodesk.AutoCAD.Runtime.CommandMethod("StreamSample", "StreamSample", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ShowSample()
            Dim editor As Editor = GetEditor()
            editor.WriteMessage(ControlChars.Lf + "====================================================" + ControlChars.Lf)
            editor.WriteMessage("Thank you for using StreamSample" + ControlChars.Lf)

            Dim options As New PromptKeywordOptions("Please select a stream type " + "[collect Bodies/collect Clip bodies/collect Materials/cUrves/Explode/eXtent/Intersect/Slice/Vector]", "collectBodies collectClipbodies collectMaterials cUrves Explode eXtent Intersect Slice Vector")

            Dim stopNow As Boolean = False
            While stopNow <> True
                Dim pr As PromptResult = editor.GetKeywords(options)
                Select Case pr.Status
                    Case PromptStatus.OK, PromptStatus.Keyword
                        DispatchCommand(pr.StringResult)
                    Case Else
                        stopNow = True
                End Select
            End While

            editor.WriteMessage(ControlChars.Lf + "====================================================" + ControlChars.Lf)
        End Sub
#End Region

#Region "HelperFunctions"
        Private Function GetDatabase() As Database
            Return Application.DocumentManager.MdiActiveDocument.Database
        End Function

        Private Function GetEditor() As Editor
            Return Application.DocumentManager.MdiActiveDocument.Editor
        End Function

        Private Function PickObject(ByVal type As Type, ByVal exactMatch As Boolean, ByVal tips As String) As ObjectId
            Dim editor As Editor = GetEditor()
            Dim options As PromptEntityOptions = Nothing
            Dim result As PromptEntityResult
            If Not (type Is Nothing) Then
                options = New PromptEntityOptions(tips)
                options.SetRejectMessage("This type of object is not allowed." + ControlChars.Lf)
                options.AddAllowedClass(type, exactMatch)
                result = editor.GetEntity(options)
            Else
                result = editor.GetEntity(tips)
            End If
            If result.Status = PromptStatus.OK And result.ObjectId.IsValid Then
                Return result.ObjectId
            End If
            Return ObjectId.Null '
        End Function

        Private Function PickObject(ByVal type As Type, ByVal exactMatch As Boolean) As ObjectId
            Return PickObject(type, exactMatch, "Please pick an object or return")
        End Function

        Private Function PickObjectSet(ByVal tips As String) As ObjectIdCollection
            Dim editor As Editor = GetEditor()
            Dim options As New PromptSelectionOptions()
            options.MessageForAdding = tips
            options.MessageForRemoval = "Remove objects from selection"
            options.AllowDuplicates = False
            options.RejectObjectsFromNonCurrentSpace = True
            Dim ids As New ObjectIdCollection()

            Dim result As PromptSelectionResult = editor.GetSelection(options)
            If result.Status = PromptStatus.OK Then
                Dim selSet As SelectionSet = result.Value
                Dim obj As SelectedObject
                For Each obj In selSet
                    ids.Add(obj.ObjectId)
                Next obj
            End If
            Return ids
        End Function

        Private Function PickObjectSet() As ObjectIdCollection
            Return PickObjectSet("Add objects to selection")
        End Function

        Private Function PromptPlane() As Plane
            Dim editor As Editor = GetEditor()
            Dim pts(3) As Point3d
            Dim numPicked As Integer = 0
            While numPicked < 3
                ' pick
                Dim result As PromptPointResult = editor.GetPoint(("Please pick point " & (numPicked + 1) & " on the plane"))
                If result.Status <> PromptStatus.OK Then
                    Return Nothing
                End If
                ' check for duplicated
                Dim pt As Point3d = result.Value
                Dim ok As Boolean = True
                For i As Integer = 0 To numPicked - 1
                    If pt.DistanceTo(pts(i)) <= Tolerance.Global.EqualPoint Then
                        editor.WriteMessage("Duplicated point, please select again." + ControlChars.Lf)
                        ok = False
                        Exit For
                    End If
                Next i
                If ok Then
                    pts(numPicked) = pt
                    numPicked += 1
                End If
            End While
            Dim v1 As Vector3d = pts(1) - pts(0)
            Dim v2 As Vector3d = pts(2) - pts(0)
            Dim normal As Vector3d = v1.CrossProduct(v2)
            Dim plane As New Plane(pts(0), normal)
            Return plane
        End Function

        Private Sub HighlightBody(ByVal body As AecModeler.Body, ByVal color As Integer)
            Dim editor As Editor = GetEditor()

            Dim edge As AecModeler.Edge
            For Each edge In body.PhysicalEdges
                editor.DrawVector(edge.Line.StartPoint, edge.Line.EndPoint, color, True)
            Next edge
        End Sub

        Private Sub HighlightBody(ByVal body As AecModeler.Body)
            HighlightBody(body, 1)
        End Sub

        Private Sub HighlightGraphics(ByVal gs As GraphicsStorage)
            Dim t As Type = gs.GetType()
            Select Case t.Name
                Case "GraphicsStorageArc"
                    HighlightGraphicsArc(CType(gs, GraphicsStorageArc))
                Case "GraphicsStorageBody"
                    HighlightGraphicsBody(CType(gs, GraphicsStorageBody))
                Case "GraphicsStorageFace"
                    HighlightGraphicsFace(CType(gs, GraphicsStorageFace))
                Case "GraphicsStorageHatch"
                    HighlightGraphicsHatch(CType(gs, GraphicsStorageHatch))
                Case "GraphicsStoragePolyline"
                    HighlightGraphicsPolyline(CType(gs, GraphicsStoragePolyline))
                Case "GraphicsStorageShell"
                    HighlightGraphicsShell(CType(gs, GraphicsStorageShell))
            End Select
        End Sub

        Private Function ArcToVectors(ByVal arc As CircularArc3d, ByVal numSegments As Integer) As Point3dCollection
            Dim interval As Interval = arc.GetInterval()
            Dim increment As Double = interval.Length / CDbl(numSegments)
            Dim controller As Double = interval.LowerBound
            Dim points As Point3dCollection = New Point3dCollection()

            For i As Integer = 0 To numSegments - 1
                points.Add(arc.EvaluatePoint(controller))
                controller += increment
            Next i

            points.Add(arc.EvaluatePoint(controller))
            Return points
        End Function

        Private Sub HighlightSegments(ByVal points As Point3dCollection, ByVal closed As Boolean)
            Dim editor As Editor = GetEditor()
            Dim maxIndex As Integer = points.Count - 1
            Dim i As Integer = 0
            While i < maxIndex
                editor.DrawVector(points(i), points(i + 1), 1, True)
                i += 1
            End While

            If closed Then editor.DrawVector(points(maxIndex), points(0), 1, True)
        End Sub

        Private Sub HighlightArc(ByVal arc As CircularArc3d)
            Dim points As Point3dCollection = ArcToVectors(arc, 25)
            HighlightSegments(points, False)
        End Sub

        Private Sub HighlightGraphicsArc(ByVal gsArc As GraphicsStorageArc)
            Dim arc3d As CircularArc3d = gsArc.Arc
            HighlightArc(arc3d)
        End Sub

        Private Sub HighlightGraphicsBody(ByVal gsBody As GraphicsStorageBody)
            HighlightBody(gsBody.Body)
        End Sub

        Private Sub HighlightGraphicsFace(ByVal gsFace As GraphicsStorageFace)
            Dim points As Point3dCollection = gsFace.Points
            HighlightSegments(points, True)
        End Sub

        Private Sub HighlightGraphicsHatch(ByVal gsHatch As GraphicsStorageHatch)
            Dim editor As Editor = GetEditor()
            Dim vectors() As LineSegment3d = gsHatch.Vectors
            For Each line As LineSegment3d In vectors
                editor.DrawVector(line.StartPoint, line.EndPoint, 1, True)
            Next line
        End Sub

        Private Sub HighlightGraphicsPolyline(ByVal gsPline As GraphicsStoragePolyline)
            Dim points As Point3dCollection = gsPline.Points
            HighlightSegments(points, False)
        End Sub

        Private Sub HighlightGraphicsShell(ByVal gsShell As GraphicsStorageShell)
            ' draw all edges including edges of holes
            Dim editor As Editor = GetEditor()
            Dim vertices As Point3dCollection = gsShell.Vertices
            Dim faceList() As Integer = gsShell.FaceList
            Dim sum As Integer = 0
            Dim count As Integer = CUInt(faceList.Length)
            While sum < count
                Dim i As Integer = sum + 1
                Dim maxIndex As Integer = CUInt(i + Math.Abs(faceList(sum)) - 1)
                Dim startPoint As Point3d = vertices(CInt(faceList(i)))
                While i < maxIndex
                    editor.DrawVector(vertices(CInt(faceList(i))), vertices(CInt(faceList(i + 1))), 1, True)
                    i += 1
                End While
                editor.DrawVector(vertices(CInt(faceList(i))), startPoint, 1, True)
                sum += CInt(Math.Abs(faceList(sum) + 1))
            End While
            Dim faceData As Autodesk.AutoCAD.GraphicsInterface.FaceData = gsShell.FaceData
            Dim edgeData As Autodesk.AutoCAD.GraphicsInterface.EdgeData = gsShell.EdgeData
        End Sub

        Private Sub HighlightBoundBox3d(ByVal box As BoundBox3d)
            If Not box.IsValid Then Exit Sub
            Dim editor As Editor = GetEditor()

            Dim width As Double = box.Width
            Dim height As Double = box.Height
            Dim depth As Double = box.Depth
            Dim minPt As Point3d = box.MinPoint

            editor.DrawVector(minPt, minPt + New Vector3d(width, 0, 0), 1, True)
            editor.DrawVector(minPt + New Vector3d(width, 0, 0), minPt + New Vector3d(width, depth, 0), 1, True)
            editor.DrawVector(minPt + New Vector3d(width, depth, 0), minPt + New Vector3d(0, depth, 0), 1, True)
            editor.DrawVector(minPt + New Vector3d(0, depth, 0), minPt, 1, True)

            editor.DrawVector(minPt + New Vector3d(0, 0, height), minPt + New Vector3d(width, 0, height), 2, True)
            editor.DrawVector(minPt + New Vector3d(width, 0, height), minPt + New Vector3d(width, depth, height), 2, True)
            editor.DrawVector(minPt + New Vector3d(width, depth, height), minPt + New Vector3d(0, depth, height), 2, True)
            editor.DrawVector(minPt + New Vector3d(0, depth, height), minPt + New Vector3d(0, 0, height), 2, True)

            editor.DrawVector(minPt, minPt + New Vector3d(0, 0, height), 3, True)
            editor.DrawVector(minPt + New Vector3d(width, 0, 0), minPt + New Vector3d(width, 0, height), 3, True)
            editor.DrawVector(minPt + New Vector3d(width, depth, 0), minPt + New Vector3d(width, depth, height), 3, True)
            editor.DrawVector(minPt + New Vector3d(0, depth, 0), minPt + New Vector3d(0, depth, height), 3, True)
        End Sub

        Private Sub HighlightProfile(ByVal profile As Profile, ByVal plane As Plane)
            Dim editor As Editor = GetEditor()
            Dim ring As Ring
            For Each ring In profile.Rings
                Dim segment As Segment2d
                For Each segment In ring.Segments
                    editor.DrawVector(New Point3d(plane, segment.StartPoint), New Point3d(plane, segment.EndPoint), 1, True)
                Next segment
            Next ring
        End Sub

        Private Sub DispatchCommand(ByVal command As String)
            Select Case command.ToLower()
                Case "collectbodies"
                    StreamCollectBodiesSample()
                Case "collectclipbodies"
                    StreamCollectClipBodiesSample()
                Case "collectmaterials"
                    StreamCollectMaterialsSample()
                Case "curves"
                    StreamCurvesSample()
                Case "explode"
                    StreamExplodeSample()
                Case "extent"
                    StreamExtentSample()
                Case "intersect"
                    StreamIntersectSample()
                Case "slice"
                    StreamSliceSample()
                Case "vector"
                    StreamVectorSample()
                Case Else
                    GetEditor().WriteMessage(ControlChars.Lf + "* Unknown Keyword *" + ControlChars.Lf)
            End Select
        End Sub
#End Region

        Private Sub StreamVectorSample()
            ' intro message
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamVector *" + ControlChars.Lf)
            editor.WriteMessage("StreamVector sample demonstrates the ability of StreamVector that breaks selected entity into primitive lines." + ControlChars.Lf)
            editor.WriteMessage("Pick an object in the current drawing and the collected lines will be highlighted." + ControlChars.Lf)

            ' pick an object
            Dim id As ObjectId = PickObject(GetType(Entity), False)
            If id.IsNull Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If
            Dim db As Database = GetDatabase()
            Dim stream As New StreamVector(db)
            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)

                ' setup the display parameters
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                ' stream the object
                stream.Stream(ent)

                stream.PopDisplayParameters()

                trans.Commit() ' for better performance
            End Using

            ' highlights the collected lines
            For Each line As LineSegment3d In stream.Lines
                editor.DrawVector(line.StartPoint, line.EndPoint, 1, True)
            Next line
        End Sub

        Private Sub StreamIntersectSample()
            ' intro message
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamIntersect *" + ControlChars.Lf)
            editor.WriteMessage("StreamIntersect detects intersections between two streams." + ControlChars.Lf)
            editor.WriteMessage("Pick two objects in the current drawing. The intersections will be highlighted and printed out in the output window." + ControlChars.Lf)

            ' pick two objects
            Dim id1 As ObjectId = PickObject(GetType(Entity), False, "Please pick the first object")
            If id1.IsNull Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim id2 As ObjectId
            Do
                id2 = PickObject(GetType(Entity), False, "Please pick the second object")
                If id2.IsNull Then
                    editor.WriteMessage("No object is picked" + ControlChars.Lf)
                    Return
                End If
            Loop While id2 = id1

            Dim db As Database = GetDatabase()
            Dim stream1 As New StreamIntersect(db, GsMarkType.None)
            Dim stream2 As New StreamIntersect(db, GsMarkType.None)
            Dim points As New Point3dCollection()

            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                Dim ent1 As Entity = CType(trans.GetObject(id1, OpenMode.ForRead), Entity)
                Dim ent2 As Entity = CType(trans.GetObject(id2, OpenMode.ForRead), Entity)

                ' put the entities in the streams respectively
                stream1.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)
                stream2.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)
                stream1.Stream(ent1)
                stream2.Stream(ent2)

                ' detect intersections
                Dim count As Integer = stream1.IntersectWith(stream2, points)

                stream1.PopDisplayParameters()
                stream2.PopDisplayParameters()

                trans.Commit()
            End Using

            ' print and highlight the results
            editor.WriteMessage((ControlChars.Lf + "Here are the intersections: " & points.Count & " in total" + ControlChars.Lf))
            Dim point As Point3d
            For Each point In points
                editor.DrawVector(point, point, 1, True)
                editor.WriteMessage(("(" & point.X & ", " & point.Y & ", " & point.Z & ")" + ControlChars.Lf))
            Next point
        End Sub

        Private Sub StreamExtentSample()
            ' intro message
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamExtent *" + ControlChars.Lf)
            editor.WriteMessage("StreamExtent detects the extents of all the objects in the stream." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing and the extents will be highlighted." + ControlChars.Lf)

            ' pick multiple objects
            Dim ids As ObjectIdCollection = PickObjectSet()
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim db As Database = GetDatabase()
            Dim stream As New StreamExtent(db)

            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            HighlightBoundBox3d(stream.Extents)
        End Sub

        Private Sub StreamExplodeSample()
            ' intro message
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamExplode *" + ControlChars.Lf)
            editor.WriteMessage("StreamExplode converts all entities to AutoCAD primitive entities." + ControlChars.Lf)
            editor.WriteMessage("Pick an object in the current drawing. The object you picked will be packaged to an anonymous block and finally referenced in the current space. The block-referenced object will be overlapping with the original one." + ControlChars.Lf)

            ' pick a object to be exploded
            Dim id As ObjectId = PickObject(GetType(Entity), False)
            If id.IsNull Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim db As Database = GetDatabase()
            Dim objects As New DBObjectCollection()
            Dim streamExplode As New StreamExplode(db, objects)
            streamExplode.IsVisualExplode = True
            streamExplode.SetForBoundarySearch(False)

            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)

                streamExplode.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)
                streamExplode.PushEntity(ent)
                streamExplode.PushProperties(ent)

                streamExplode.Stream(ent)
                streamExplode.PackageExplodedEntities()

                streamExplode.PopProperties()
                streamExplode.PopEntity()
                streamExplode.PopDisplayParameters()

                trans.Commit()
            End Using

            Dim blockRef As BlockReference = Nothing
            For Each obj As DBObject In objects
                If TypeOf obj Is BlockReference Then
                    blockRef = CType(obj, BlockReference)
                    Exit For
                End If
            Next obj

            If blockRef Is Nothing Then Return

            blockRef.Position = Point3d.Origin

            Using trans As Transaction = tm.StartTransaction()
                Dim btr As BlockTableRecord = CType(trans.GetObject(db.CurrentSpaceId, OpenMode.ForWrite), BlockTableRecord)
                btr.AppendEntity(blockRef)
                trans.AddNewlyCreatedDBObject(blockRef, True)
                trans.Commit()
            End Using
        End Sub

        Private Sub StreamCurvesSample()
            Dim editor As Editor = GetEditor()
            Dim db As Database = GetDatabase()
            editor.WriteMessage("* StreamCurves *" + ControlChars.Lf)
            editor.WriteMessage("StreamCurves collects curves and lines from the objects passed in." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing. The collected graphics will be highlighted while the numbers of collected lines and curves will be printed in the output window." + ControlChars.Lf)

            Dim ids As ObjectIdCollection = PickObjectSet()
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim stream As New StreamCurves(db)
            ' collect arcs instead of breaking arcs into line segments
            stream.CollectArcs = True
            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            Dim lines As LineSegment3d() = stream.Lines
            Dim arcs As CircularArc3d() = stream.Arcs
            editor.WriteMessage(("Collected " & lines.Length & " lines and " & arcs.Length & " arcs." & ControlChars.Lf))
            For Each line As LineSegment3d In lines
                editor.DrawVector(line.StartPoint, line.EndPoint, 1, True)
            Next line
            For Each arc As CircularArc3d In arcs
                HighlightArc(arc)
            Next arc
        End Sub

        Private Sub StreamCollectBodiesSample()
            Dim db As Database = GetDatabase()
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamCollectBodies *" + ControlChars.Lf)
            editor.WriteMessage("StreamCollectBodies collects bodies from the objects passed in." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing. The collected bodies will be highlighted in the current view. And the volume of the bodies will be printed out in the output window." + ControlChars.Lf)

            Dim ids As ObjectIdCollection = PickObjectSet()
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim stream As New StreamCollectBodies(db)
            Dim tm As TransactionManager = db.TransactionManager

            Using trans As Transaction = tm.StartTransaction()
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            ' draw the collected body and output the volume of the body
            Dim body As AecModeler.Body = stream.GetBodyCopy()
            HighlightBody(body)

            editor.WriteMessage(("Body volume = " + body.Volume.ToString() + ControlChars.Lf))
        End Sub

        Private Sub StreamSliceSample()
            Dim db As Database = GetDatabase()
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamSlice *" + ControlChars.Lf)
            editor.WriteMessage("StreamSlice slices bodies with a plane." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing to be sliced and then pick three points to define the plane. The result profile will be highlighted." + ControlChars.Lf)
            Dim ids As ObjectIdCollection = PickObjectSet("Please pick the bodies need to be sliced")
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            editor.WriteMessage("Now you need to pick 3 points from the screen to define the slicing plane." + ControlChars.Lf)
            Dim plane As Plane = PromptPlane()
            If plane Is Nothing Then
                Return
            End If
            Dim stream As New StreamSlice(db, plane)

            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            HighlightProfile(stream.GetProfile(), plane)
        End Sub

        Private Sub StreamCollectClipBodiesSample()
            Dim db As Database = GetDatabase()
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamCollectClipBodies *" + ControlChars.Lf)
            editor.WriteMessage("StreamCollectClipBodies clips all geometry pushed in against the supplied body." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing and then pick an mass element to define the clipping boundary. The collected graphics will be highlighted in the current view." + ControlChars.Lf)

            Dim ids As ObjectIdCollection = PickObjectSet("Please pick the objects to be clipped")
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim massElemId As ObjectId = PickObject(GetType(MassElement), True, "Please pick a mass element to define the clipping boundary")
            If massElemId.IsNull Then
                editor.WriteMessage("A mass element is needed to define the clipping boundary." + ControlChars.Lf)
                Return
            End If

            Dim stream As New StreamCollectClipBodies(db)
            ' You may tell the stream to retain bodies instead of turning bodies into shells.
            ' stream.SetRetainBodies(true);
            ' But now we use the default setting, which uses shell as output.
            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                Dim masselem As MassElement = CType(trans.GetObject(massElemId, OpenMode.ForRead), MassElement)
                Dim body As AecModeler.Body = masselem.Body.Transform(masselem.Ecs)
                stream.SetBodyClipVolume(body)

                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            Dim gsCollection As GraphicsStorage() = stream.GetCollectedGraphics()
            Dim gs As GraphicsStorage
            For Each gs In gsCollection
                HighlightGraphics(gs)
            Next gs
        End Sub

        Private Sub StreamCollectMaterialsSample()
            Dim db As Database = GetDatabase()
            Dim editor As Editor = GetEditor()
            editor.WriteMessage("* StreamCollectMaterials *" + ControlChars.Lf)
            editor.WriteMessage("StreamCollectMaterials collects bodies of the specified materials from the entities pushed into the stream." + ControlChars.Lf)
            editor.WriteMessage("Pick some objects in the current drawing. The bodies with materials specifed will be highlighted. And bodies with the same material are displayed in one color." + ControlChars.Lf)

            Dim stream As New StreamCollectMaterials(db)
            ' StreamCollectMaterials needs the users to specify the materials they're interested in by setting the MaterialFilter property.
            ' Here we add all the material ids in the current drawing to the filter collection.
            Dim dictMaterials As New DictionaryMaterialDefinition(db)
            stream.MaterialFilter = dictMaterials.Records
            ' To combine the bodies with the same material instead of traversing the body chain
            stream.CombineBodies = True

            Dim ids As ObjectIdCollection = PickObjectSet()
            If ids.Count = 0 Then
                editor.WriteMessage("No object is picked" + ControlChars.Lf)
                Return
            End If

            Dim tm As TransactionManager = db.TransactionManager
            Using trans As Transaction = tm.StartTransaction()
                stream.PushDisplayParameters(DictionaryDisplayConfiguration.GetStandardDisplayConfiguration(db), trans)

                For Each id As ObjectId In ids
                    Dim ent As Entity = CType(trans.GetObject(id, OpenMode.ForRead), Entity)
                    stream.Stream(ent)
                Next id

                stream.PopDisplayParameters()
                trans.Commit()
            End Using

            ids = stream.MaterialIds
            Dim color As Integer = 1
            For Each id As ObjectId In ids
                Dim body As AecModeler.Body = stream.GetBody(id) '
                HighlightBody(body, color)
                color += 1
            Next id
        End Sub
    End Class
End Namespace