#Region " Autodesk Copyright "

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region "Namespaces References"

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.Windows.Forms
Imports System.Drawing

' AutoCAD specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices

' AutoCAD MEP Specific
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices
Imports MepUtilities

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(CatalogDifference.CatalogDifferenceSample))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(CatalogDifference.CmdCatalogDifference))> 

Namespace CatalogDifference
    Public Class CatalogDifferenceSample
        Implements IExtensionApplication

#Region "IExtensionApplication"
        Public Sub Initialize() Implements IExtensionApplication.Initialize
            Dim ed As Autodesk.AutoCAD.EditorInput.Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            ed.WriteMessage("Loading Catalog Difference Sample..." & vbCrLf)
            ed.WriteMessage("Type ""Catalog Difference"" to run the sample..." & vbCrLf)
        End Sub

        Public Sub Terminate() Implements IExtensionApplication.Terminate
        End Sub

#End Region

    End Class


    Public Class CmdCatalogDifference

        Public mTrans As Transaction = Nothing

#Region "CatalogDifference Part Command"

        ' Sample application to show the differences between a database part
        ' and the part defined in the catalog
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("CatalogDifference", "CatalogDifference", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub CatDiff()

            Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
            Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

            ' start a new transaction
            '
            mTrans = transMgr.StartTransaction()

            Try
                Dim prompt As String = Constants.vbLf & "Select a member to show the difference between database and catalog: "

                Dim oidSelected As ObjectId

                oidSelected = MepUtilities.PromptingUtilites.PromptForObject(prompt, Nothing, GetType(Member), False)

                If oidSelected.IsNull Then
                    Return
                End If


                ' open the member for read based on the objectId
                '
                Dim member As Member = CType(transMgr.GetObject(oidSelected, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForRead, False), Member)

                If Not member Is Nothing Then

                    Dim frm As CatalogDifferenceDlg = New CatalogDifferenceDlg(member)

                    AcadApplication.ShowModalDialog(frm)

                    mTrans.Commit()

                End If

            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException

                If be.ErrorCode = ErrorCode.QueryResultNull Then
                    MessageBox.Show("The query did not find any parts.")
                End If

                Debug.WriteLine(be.Message)

                mTrans.Commit()

            Catch e As SystemException

                Debug.WriteLine(e.Message)

                mTrans.Commit()

            Finally

                mTrans.Dispose()

            End Try
        End Sub

#End Region

    End Class

End Namespace
