#Region " Autodesk Copyright "

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId

Imports MepUtilities


Namespace CatalogDifference

    Partial Public Class CatalogDifferenceDlg
        Inherits Form

        Private mbCatalogViewActive As Boolean = True
        Private mMember As Member
        Private mCatalogDataRecord As DataRecord 'Data record representing part data from the catalog
        Private mMemberDataRecord As DataRecord  'Data record representing part data from the object in the drawing

        Public Sub New(ByVal mMember As Member)

            Me.mMember = mMember
            InitializeComponent()
            mMemberDataRecord = PartManager.GetPartData(mMember) 'This method loads the part data from the object in the drawing
            GetCatalogData() 'This method loads the part data from the catalog.

        End Sub

        Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load

            LoadPartListView()
            LoadCatalogListView()

        End Sub

        Private Sub lvCatalog_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles lvCatalog.SelectedIndexChanged

            Try

                If mbCatalogViewActive AndAlso lvCatalog.SelectedItems.Count > 0 Then
                    Dim index As Integer = lvCatalog.SelectedItems(0).Index
                    Dim item As ListViewItem = lvPartData.FindItemWithText(lvCatalog.Items(index).Text)
                    item.Selected = True
                    item.EnsureVisible()
                End If

            Catch

            End Try

        End Sub

        Private Sub lvPartData_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles lvPartData.SelectedIndexChanged

            Try

                If (Not mbCatalogViewActive) AndAlso lvPartData.SelectedItems.Count > 0 Then

                    Dim index As Integer = lvPartData.SelectedItems(0).Index
                    Dim item As ListViewItem = lvCatalog.FindItemWithText(lvCatalog.Items(index).Text)
                    item.Selected = True
                    item.EnsureVisible()

                End If

            Catch

            End Try

        End Sub
        Private Sub toolStripButton1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles toolStripButton1.Click

            ' update the part            
            Try

                If MessageBox.Show("Updating the part will update all parts in the drawing that use the same style. " & Constants.vbLf & "Do you want to continue?", "Change all parts", MessageBoxButtons.YesNo) = Windows.Forms.DialogResult.Yes Then

                    ' Upgrade the object to Write
                    '
                    mMember.UpgradeOpen()

                    ' set the new part data
                    '
                    PartManager.SetPartData(mMember, mCatalogDataRecord)

                    ' downgrade it back to Read
                    '
                    mMember.DowngradeOpen()

                    ' get the new data
                    '
                    mMemberDataRecord = PartManager.GetPartData(mMember)

                    ' reload the UI
                    '
                    LoadPartListView()
                    LoadCatalogListView()

                End If

            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException

                MessageBox.Show(be.Message)

            End Try

        End Sub

        Private Sub LoadCatalogListView()

            lvCatalog.Items.Clear()

            Dim matchValue As String

            Dim extentName As Integer = 0
            Dim extentValue As Integer = 0

            toolStripButton1.Enabled = False

            lblCatData.Text = "Contains " & mCatalogDataRecord.DataFields.Count & " data fields"

            For Each datafield As DataField In mCatalogDataRecord.DataFields

                Dim value As String = MepUtilities.DataUtilities.DataFieldToString(datafield)
                Dim items As String() = New String() {datafield.Description, value}
                Dim lvItem As ListViewItem = New ListViewItem(items)
                lvItem.Name = datafield.Description
                If (Not lvPartData.Items.ContainsKey(datafield.Description)) Then
                    lvItem.BackColor = System.Drawing.Color.Green
                    lvItem.ForeColor = System.Drawing.Color.White
                    Dim curFont As Font = New Font(lvItem.Font, FontStyle.Bold)
                    lvItem.Font = curFont
                    lvItem.Checked = True
                    toolStripButton1.Enabled = True
                Else
                    Dim lvItems As ListViewItem() = lvPartData.Items.Find(datafield.Description, True)
                    If lvItems.Length = 1 Then ' could be more than one, need to check connections
                        Dim item As ListViewItem = lvItems(0)
                        matchValue = item.SubItems(1).Text
                        If matchValue <> value Then
                            lvItem.BackColor = System.Drawing.Color.Red
                            lvItem.ForeColor = System.Drawing.Color.White
                            Dim curFont As Font = New Font(lvItem.Font, FontStyle.Bold)
                            lvItem.Font = curFont
                            lvItem.Checked = True
                            toolStripButton1.Enabled = True
                        End If
                    End If
                End If

                FindMaxSizes(extentName, extentValue, lvItem)

                lvCatalog.Columns(0).Width = extentName + 10
                lvCatalog.Columns(1).Width = extentValue + 10
                lvItem.Tag = datafield
                lvCatalog.Items.Add(lvItem)

            Next datafield

        End Sub

        Private Sub LoadPartListView()

            lvPartData.Items.Clear()

            Dim extentNamePD As Integer = 0
            Dim extentValuePD As Integer = 0

            lblPartData.Text = "Contains " & mMemberDataRecord.DataFields.Count & " data fields"

            For Each dataField As DataField In mMemberDataRecord.DataFields
                Dim items As String() = New String() {dataField.Description, MepUtilities.DataUtilities.DataFieldToString(dataField)}
                Dim lvItem As ListViewItem = New ListViewItem(items)
                lvItem.Name = dataField.Description

                FindMaxSizes(extentNamePD, extentValuePD, lvItem)

                lvPartData.Columns(0).Width = extentNamePD + 10
                lvPartData.Columns(1).Width = extentValuePD + 10

                lvItem.Tag = dataField
                lvPartData.Items.Add(lvItem)

            Next dataField

        End Sub

        Private Sub FindMaxSizes(ByRef extentNamePD As Integer, ByRef extentValuePD As Integer, ByVal lvItem As ListViewItem)

            Dim g As Graphics = Me.CreateGraphics()
            Dim sizeName As SizeF = g.MeasureString(lvItem.Text, Font)
            Dim sizeValue As SizeF = g.MeasureString(lvItem.SubItems(1).ToString(), Font)

            If sizeName.Width > extentNamePD Then
                extentNamePD = CInt(Fix(sizeName.Width))
            End If

            If sizeValue.Width > extentValuePD Then
                extentValuePD = CInt(Fix(sizeValue.Width))
            End If

        End Sub

        Private Sub lvCatalog_Enter(ByVal sender As Object, ByVal e As EventArgs) Handles lvCatalog.Enter
            ' need this so selecting items is not recursive
            mbCatalogViewActive = True
        End Sub

        Private Sub lvPartData_Enter(ByVal sender As Object, ByVal e As EventArgs) Handles lvPartData.Enter

            ' need this so selecting items is not recursive
            mbCatalogViewActive = False

        End Sub

        Private Sub GetCatalogData()

            Try
                'We already have the part catalog data from the object in the drawing.  We can use
                'the object to populate the query before calling part manager.
                'Create the query object and fill it in with new parameters

                Dim query As DataQuery = New DataQuery()

                query.Domain = mMember.Domain

                Try
                    'Let's use the guid from the existing object since we already know which one it is.
                    query.PartGuid = PartManager.GetPartGuid(mMember)
                Catch ex As Autodesk.AutoCAD.Runtime.Exception
                    If ex.ErrorStatus = ErrorStatus.KeyNotFound Then
                        MessageBox.Show("GUID not found in datafield")
                    End If
                End Try

                'Each part can have multiple size combinations.  When we query the catalog, we
                'have the ability to query for a particular size.  In this case, we already have the 
                'exact size information stored in the member data record.  
                'Technically, we could add every data field present in the data record to the query.
                'However, that will be overkill.  So, we will only add data field's whose raw data source
                'is basic table or constant lists.  Size information such as diameter etc. tend to use these
                'data sources and since we are trying to narrow down the size combinations, using them will be
                'most effective.

                Dim count As Integer = mMemberDataRecord.DataFields.Count

                For Each datafield As DataField In mMemberDataRecord.DataFields

                    ' use only Basic Table and ConstList here
                    '
                    If datafield.RawDataSource = DataSource.BasicTable OrElse datafield.RawDataSource = DataSource.ConstantLists Then
                        MepUtilities.DataUtilities.AddDataFieldToQuery(datafield, query)
                    End If

                Next datafield

                'Set the maximum number of records to one.  
                'Since we already added size information to our query, when we run the query we 
                'should exactly one record but for the purposes of this sample, let's set the value to 1.
                '
                Dim maxRecords As Integer = 1

                ' now get the ExpandedTable from our query
                '
                Dim exTable As DataExpandedTable = PartManager.GetPartTable(query, maxRecords)

                ' now get all the records from the table
                '
                Dim recordCol As DataRecordCollection = exTable.DataRecords
                If recordCol.Count > 0 Then
                    mCatalogDataRecord = recordCol(0)
                End If

            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException

                If be.ErrorCode = ErrorCode.QueryResultNull Then
                    MessageBox.Show("Part no longer contain in the catalog.")
                End If

            End Try

        End Sub

    End Class
End Namespace