#Region "Autodesk Copyright"

' .NET Sample 
' 
' Copyright (c) 2008 by Autodesk, Inc. 
' 
' Permission to use, copy, modify, and distribute this software 
' for any purpose and without fee is hereby granted, provided 
' that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and 
' restricted rights notice below appear in all supporting 
' documentation. 
' 
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
' UNINTERRUPTED OR ERROR FREE. 
' 
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer 
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
' (Rights in Technical Data and Computer Software), as applicable. 
' 

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
'using System.Windows.Forms; 
'using System.Drawing; 

' AutoCAD MEP Specific 
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.GraphicsSystem
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices

' create an alias to the AutoCAD Application object 
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports DbObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(ConnectionManagerSample.ConnectionManagerSample))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(ConnectionManagerSample.ConnectionManagerSample))> 

Namespace ConnectionManagerSample
    Public Class ConnectionManagerSample
        Implements IExtensionApplication
#Region "Member Data"

        Private mEditor As Editor
        Private mDb As Database
        Private mDoc As Document
        Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager

        Private oidParts As ObjectIdCollection

        Private mDuctDataRecord As DataRecord
        Private mFittingDataRecord As DataRecord
        Private mDuctExpandedTable As DataExpandedTable
        Private mFittingExpandedTable As DataExpandedTable
#End Region

#Region "IExtensionApplication - Intialize/Terminate"
        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.Initialize </name>
        ''' <returntype> void </returntype>
        ''' <summary>Initialize the application extension,  including adding sample
        ''' parts to the drawing and writing the supported commands to the command window</summary>
        ''' </member>
        Public Sub Initialize() Implements IExtensionApplication.Initialize
            oidParts = New ObjectIdCollection()
            AddParts()
            mEditor.WriteMessage(vbNewLine & "Added the following commands: ")
            mEditor.WriteMessage(vbNewLine & "GetConnectedObjects")
            mEditor.WriteMessage(vbNewLine & "IsBranchObject")
            mEditor.WriteMessage("" & Chr(10) & "SelectBranch")
            mEditor.WriteMessage("" & Chr(10) & "GetNumberOfRuns")
            mEditor.WriteMessage("" & Chr(10) & "GetStartObject")
            mEditor.WriteMessage("" & Chr(10) & "HighLightRuns")
        End Sub

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.Terminate </name>
        ''' <returntype> void </returntype>
        ''' <summary>Terminate the application extension.  In this case we 
        ''' simply remove the sample parts that we created during initialize</summary>
        ''' </member>

        Public Sub Terminate() Implements IExtensionApplication.Terminate
            RemoveParts()
        End Sub
#End Region

#Region "Setup"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.Setup </name>
        ''' <returntype> void </returntype>
        ''' <summary>Setup the extension application by getting references
        ''' to frequently used objects</summary>
        ''' </member>

        Public Sub Setup()
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mDoc = AcadApplication.DocumentManager.MdiActiveDocument
            mTransMgr = mDb.TransactionManager
        End Sub
#End Region

#Region "AddParts"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.AddParts </name>
        ''' <returntype> void </returntype>
        ''' <summary>This method adds a number of parts to the drawing to 
        ''' create a sample duct run that can be used with the commands in 
        ''' this class</summary>
        ''' </member>

        Public Sub AddParts()
            Setup()
            mEditor.WriteMessage("" & Chr(10) & "Adding parts for sample...")
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    oidParts.Add(AddMvPart(New Point3d(0, 0, 0), "F6882D30-3F59-49DD-9024-7D51BD277D80"))
                    oidParts.Add(AddDuct(New Point3d(16, -2, 13), New Point3d(16, -145.99543, 13)))
                    oidParts.Add(AddFitting(New Point3d(16, -145.99543, 13), AngleToRadians(270), 120, "0D97CE03-2B06-45EC-8FF1-9497611D3D51", True))
                    oidParts.Add(AddDuct(New Point3d(22, -156.38773, 13), New Point3d(141.75381, -225.52763, 13)))
                    oidParts.Add(AddFitting(New Point3d(141.75381, -225.52763, 13), AngleToRadians(330), 60, "3A2D3DFE-55E5-4D15-B2C9-865B2C6881ED", False))
                    oidParts.Add(AddDuct(New Point3d(146.48586, -233.72378, 13), New Point3d(146.48586, -477.6875, 13)))
                    oidParts.Add(AddDuct(New Point3d(10, -156.38773, 13), New Point3d(-109.97548, -225.65561, 13)))
                    oidParts.Add(AddFitting(New Point3d(-114.70753, -233.85176, 13), AngleToRadians(90), 60, "3A2D3DFE-55E5-4D15-B2C9-865B2C6881ED", False))
                    oidParts.Add(AddDuct(New Point3d(-114.70753, -233.85176, 13), New Point3d(-114.70753, -451.6875, 13)))


                    mDoc.SendStringToExecute(".zoom _e ", True, False, False)
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
            mEditor.WriteMessage("done.")
        End Sub
#End Region

#Region "RemoveParts"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.RemoveParts </name>
        ''' <returntype> void </returntype>
        ''' <summary>Remove the sample parts that were added in AddParts</summary>
        ''' </member>

        Public Sub RemoveParts()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Erase each part in the part collection
                    For Each id As ObjectId In oidParts
                        Dim obj As DbObject = mTransMgr.GetObject(id, OpenMode.ForWrite, False, False)
                        obj.[Erase]()
                    Next
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "Command GetConnectedObjects"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdGetConnectedObjects </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command prompts the user to select an object, then highlights 
        ''' all objects connected to the selected object</summary>
        ''' </member>

        <CommandMethod("GetConnectedObjects", CommandFlags.Modal)> _
        Public Sub CmdGetConnectedObjects()
            Setup()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Ask the user to select an object
                    Dim oidSelected As ObjectId = PromptForObject("Select object:", "", GetType(Member), False)
                    If Not oidSelected.IsNull Then
                        ' Find objects connected to the selected one
                        Dim oidCollection As ObjectIdCollection = ConnectionManager.GetAllConnectedObjects(oidSelected)
                        For Each oid As ObjectId In oidCollection
                            ' Highlight this object
                            Dim ent As Member = DirectCast(OpenObject(oid, OpenMode.ForWrite), Member)
                            ent.Highlight()
                        Next
                    End If
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "Command IsBranchObject"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdIsBranchObject </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command prompts the user to select an object,  then writes to the
        ''' command line if the object is a branch object or not</summary>
        ''' </member>

        <CommandMethod("IsBranchObject", CommandFlags.Modal)> _
        Public Sub CmdIsBranchObject()
            Setup()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Ask the user to select an object
                    Dim oidSelected As ObjectId = PromptForObject("" & Chr(10) & "Select object:", "", GetType(Member), False)
                    If Not oidSelected.IsNull Then
                        ' Check if the select object is a branch
                        Dim branch As Boolean = ConnectionManager.IsBranch(oidSelected)
                        If branch Then
                            mEditor.WriteMessage("Selected object is branch.")
                        Else
                            mEditor.WriteMessage("Selected object is not branch.")
                        End If
                    End If
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "Command SelectBranch"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdSelectBranch </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command prompts the user for an object,  then highlights the
        ''' entire run</summary>
        ''' </member>

        <CommandMethod("SelectBranch", CommandFlags.Modal)> _
        Public Sub CmdSelectBranch()
            Setup()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Prompt the user to select an object
                    Dim oidSelected As ObjectId = PromptForObject("" & Chr(10) & "Select object:", "", GetType(Member), False)
                    If Not oidSelected.IsNull Then
                        ' Get the entire connected run
                        Dim oidcollection As ObjectIdCollection = ConnectionManager.GetRun(oidSelected)
                        For Each oid As ObjectId In oidcollection
                            ' Highlight this object
                            Dim ent As Member = DirectCast(OpenObject(oid, OpenMode.ForRead), Member)
                            ent.Highlight()
                        Next
                    End If
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "Command GetNumberOfRuns"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdGetNumberOfRuns </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command gets the total number of runs in the current drawing and 
        ''' writes it to the command line</summary>
        ''' </member>

        <CommandMethod("GetNumberOfRuns", CommandFlags.Modal)> _
        Public Sub CmdGetNumberOfRuns()
            Setup()
            Try
                mEditor.WriteMessage("" & Chr(10) & "Number of runs: " + ConnectionManager.GetNumberOfRunsInDatabase(mDb).ToString())
            Catch ex As System.Exception
                mEditor.WriteMessage(ex.Message)
            End Try
        End Sub
#End Region

#Region "Command GetStartObject"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdGetStartObject </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command prompts the user to select an object,  then highlights
        ''' the first object in the connected run</summary>
        ''' </member>

        <CommandMethod("GetStartObject", CommandFlags.Modal)> _
        Public Sub CmdGetStartObject()
            Setup()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Prompt the user to select an object
                    Dim oidSelected As ObjectId = PromptForObject("Select object:", "", GetType(Member), False)
                    If Not oidSelected.IsNull Then
                        ' Find the start object for the run
                        Dim oidStart As ObjectId = ConnectionManager.FindStartObject(oidSelected)
                        If Not oidStart.IsNull Then
                            ' Highlight this object
                            Dim mem As Member = DirectCast(OpenObject(oidStart, OpenMode.ForRead), Member)
                            mem.Highlight()
                        End If
                    End If
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "Command HighLightRuns"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.CmdHighLightRuns </name>
        ''' <returntype> void </returntype>
        ''' <summary>This command highlights the runs in the current drawing,  one run at 
        ''' a time.  The user can choose Next or Previous to cycle through the runs.</summary>
        ''' </member>

        <CommandMethod("HighLightRuns", CommandFlags.Modal)> _
        Public Sub CmdHighLightRuns()
            Setup()
            Using trans As Transaction = mTransMgr.StartTransaction()
                Try
                    ' Find the total number of runs
                    Dim numberOfRuns As Integer = ConnectionManager.GetNumberOfRunsInDatabase(mDb)
                    Dim ps As PromptStatus = PromptStatus.OK
                    Dim index As Integer = 0
                    While ps = PromptStatus.OK
                        ' Highlight the currently selected run
                        Dim oids As ObjectIdCollection = ConnectionManager.GetRunFromDatabase(mDb, index)
                        HighlightEntities(oids)

                        ' Set up the prompt that allows the user to cycle through the runs
                        Dim prOpt As New PromptKeywordOptions("" & Chr(10) & "Select")
                        prOpt.Keywords.Add("Next")
                        prOpt.Keywords.Add("Previous")
                        prOpt.Keywords.Add("End")
                        prOpt.AllowArbitraryInput = False
                        prOpt.AllowNone = False

                        ' Prompt the user
                        Dim pr As PromptResult = mEditor.GetKeywords(prOpt)
                        ps = pr.Status
                        If ps = PromptStatus.OK Then
                            Dim result As String = pr.StringResult
                            If result = "Next" Then
                                ' Move to the next run
                                index += 1
                                prOpt.Message = "" & Chr(10) & "Select Next/Previous/End: <N>"
                            ElseIf result = "Previous" Then
                                ' Move to the previous run
                                index -= 1
                                prOpt.Message = "" & Chr(10) & "Select Next/Previous/End: <P>"
                            Else
                                ps = PromptStatus.Cancel
                            End If
                        End If

                        ' Make sure we wrap around to the other end of the set of runs
                        If index >= numberOfRuns Then
                            index = 0
                        End If
                        If index < 0 Then
                            index = numberOfRuns - 1
                        End If
                        ' Unhighlight the currently highlighted objects
                        UnHighlightEntities(oids)
                    End While
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message)
                Finally
                    trans.Commit()
                End Try
            End Using
        End Sub
#End Region

#Region "HighlightEntities"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.HighlightEntities </name>
        ''' <returntype> void </returntype>
        ''' <summary>Highlight each object in the given collection</summary>
        ''' <param name="oids"></param>
        ''' </member>

        Public Sub HighlightEntities(ByVal oids As ObjectIdCollection)
            For Each id As ObjectId In oids
                Dim ent As Entity = DirectCast(OpenObject(id, OpenMode.ForRead), Entity)
                ent.Highlight()
            Next
        End Sub
#End Region

#Region "UnHighlightEntities"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.UnHighlightEntities </name>
        ''' <returntype> void </returntype>
        ''' <summary>Unhighlight each object in the given collection</summary>
        ''' <param name="oids"></param>
        ''' </member>

        Public Sub UnHighlightEntities(ByVal oids As ObjectIdCollection)
            For Each id As ObjectId In oids
                Dim ent As Entity = DirectCast(OpenObject(id, OpenMode.ForRead), Entity)
                ent.Unhighlight()
            Next
        End Sub
#End Region

#Region "OpenObject"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.OpenObject </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.DBObject </returntype>
        ''' <summary>Open the object corresponding to the given object id</summary>
        ''' <param name="oid">The object id of the object to be opened</param>
        ''' <param name="mode">The open mode to use</param>
        ''' </member>

        Public Function OpenObject(ByVal oid As ObjectId, ByVal mode As OpenMode) As Autodesk.AutoCAD.DatabaseServices.DBObject
            Try
                Dim dbObj As Autodesk.AutoCAD.DatabaseServices.DBObject
                ' Before opening the given object,  make sure we open the Block Table Record 
                Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
                Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), mode, False), BlockTableRecord)
                ' Now open the object
                dbObj = mTransMgr.GetObject(oid, OpenMode.ForWrite, False)
                Return dbObj
            Catch
                Return Nothing
            End Try
        End Function
#End Region

#Region "PromptForObject"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.PromptForObject </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select an object of the specified type</summary>
        ''' <param name="prompt">String to use to prompt the user</param>
        ''' <param name="rejectMessage">String to use when the user selects an object of the wrong type</param>
        ''' <param name="objType">Type of object that the user is to select</param>
        ''' <param name="exactMatch"></param>
        ''' </member>

        Public Shared Function PromptForObject(ByVal prompt As String, ByVal rejectMessage As String, ByVal objType As System.Type, ByVal exactMatch As Boolean) As ObjectId
            ' Set up the prompt options
            Dim prOptions As New PromptEntityOptions("")
            If prompt Is Nothing OrElse prompt.Length = 0 Then
                prOptions.Message = "" & Chr(10) & "Select object: "
            Else
                prOptions.Message = prompt
            End If

            If rejectMessage Is Nothing OrElse rejectMessage.Length = 0 Then
                prOptions.SetRejectMessage(vbNewLine & "Object must be of type " + objType.ToString())
            Else
                prOptions.SetRejectMessage(rejectMessage)
            End If

            prOptions.AddAllowedClass(objType, exactMatch)

            Try
                ' prompt the user to select a member 
                ' 
                Dim prResult As PromptEntityResult = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor.GetEntity(prOptions)

                If prResult.Status = PromptStatus.Cancel Then
                    Return Autodesk.AutoCAD.DatabaseServices.ObjectId.Null
                End If

                ' get the ObjectId from the selected member 
                ' 
                Return prResult.ObjectId
            Catch ex As SystemException
                Throw ex
            End Try
        End Function

#End Region

#Region "AddFitting"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.AddFitting </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Adds a fitting to the drawing</summary>
        ''' <param name="ptStart">The location to place the fitting</param>
        ''' <param name="rotation">The rotation at which to place the fitting</param>
        ''' <param name="angle">The angle to use,  for parts that come in 
        ''' different angles.  This is commonly used with elbows</param>
        ''' <param name="partGuid">The guid that identifies the catalog part to insert</param>
        ''' <param name="usePathAngle">Is the angle needed when specifying the part size</param>
        ''' </member>

        Private Function AddFitting(ByVal ptStart As Point3d, ByVal rotation As Double, ByVal angle As Double, ByVal partGuid As String, ByVal usePathAngle As Boolean) As ObjectId
            ' Create a generic duct fitting
            Dim fitting As New DuctFitting()
            fitting.SetToStandard(mDb)
            fitting.SetDatabaseDefaults(mDb)
            ' Create the specific fitting from the catalog
            SetFittingPartData(fitting, angle, partGuid, usePathAngle)
            fitting.Location = ptStart
            fitting.Rotation = rotation

            ' now set the layer for the object 
            fitting.SetDefaultLayer()

            ' finally we will add the duct to the database ands commit the transaction 
            Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
            Dim oid As ObjectId = btr.AppendEntity(fitting)
            mTransMgr.AddNewlyCreatedDBObject(fitting, True)
            Return oid
        End Function
#End Region

#Region "AddDuct"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.AddDuct </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>This method adds a duct to the drawing,  from ptStart to ptEnd</summary>
        ''' <param name="ptStart">The start point of the duct</param>
        ''' <param name="ptEnd">The end point of the duct</param>
        ''' </member>

        Private Function AddDuct(ByVal ptStart As Point3d, ByVal ptEnd As Point3d) As ObjectId
            ' create a new Duct object 
            ' 
            Dim duct As New Duct()

            ' make sure to call these before setting your geometry on the part 
            duct.SetToStandard(mDb)
            duct.SetDatabaseDefaults(mDb)
            SetDuctPartData(duct)

            ' set the line segment onto the duct 
            ' 
            Dim lineSeg As New LineSegment3d(ptStart, ptEnd)
            duct.SetBaseCurve(lineSeg, Vector3d.ZAxis)

            ' now set the layer for the object 
            ' 
            duct.SetDefaultLayer()

            ' finally we will add the duct to the database ands commit the transaction 
            Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
            Dim oid As ObjectId = btr.AppendEntity(duct)
            mTransMgr.AddNewlyCreatedDBObject(duct, True)
            Return oid
        End Function
#End Region

#Region "SetDuctPartData"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.SetDuctPartData </name>
        ''' <returntype> void </returntype>
        ''' <summary>This method sets up the duct to correspond to a specific duct from
        ''' the catalog.  It uses a 12"x12" rectangular duct</summary>
        ''' <param name="duct"></param>
        ''' </member>

        Private Sub SetDuctPartData(ByVal duct As Duct)
            ' the next bit of code shows how to query the catalog for a part 
            ' First we create a query object, then add the parameters that we are interested in 
            ' to the query. Based on the amount of parameters specified, the results of the query 
            ' may contain multiple records, one record, or no records. 

            If mDuctExpandedTable Is Nothing Then
                Dim width As Double = 12
                Dim height As Double = 12

                Dim query As New DataQuery()
                ' we want a duct 
                query.Domain = Domain.DuctComponent
                ' specify a rectangular duct 
                query.PartGuid = "352B04D8-B46F-4578-8C93-82F5A0D11806"
                ' Specify a 12" width 
                query.AddSizeParameter(Context.ConnectionSizeRectangularWidth, 1, width)
                ' Specify a 12" height 
                query.AddSizeParameter(Context.ConnectionSizeRectangularHeight, 1, height)
                Dim formatter As New EnumStringFormatter()
                Dim strValue As String = [String].Format(formatter, "{0:CatG}", ConnectionType.Slip)
                query.AddSizeParameter(Context.ConnectionPortPhysicalType, 1, strValue) ' of type Slip 

                ' set the maximum number of records we would like to see 
                Dim maxRecords As Integer = 10

                ' get the ExpandedTable from our query 
                mDuctExpandedTable = PartManager.GetPartTable(query, maxRecords)

                ' get all the records from the table 
                Dim recordCol As DataRecordCollection = mDuctExpandedTable.DataRecords

                mDuctDataRecord = recordCol(0)
            End If

            ' this will add the catalog part data to the database object 
            ' 
            PartManager.CreatePartViaRecord(mDuctExpandedTable, mDuctDataRecord, duct)
        End Sub
#End Region

#Region "SetFittingPartData"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.SetFittingPartData </name>
        ''' <returntype> void </returntype>
        ''' <summary>Set up the given fitting to correspond to a specific fitting from the
        ''' catalog.</summary>
        ''' <param name="fitting">The fitting to be set up</param>
        ''' <param name="angle">The path angle</param>
        ''' <param name="partGuid">The catalog part guid</param>
        ''' <param name="has3rdConnector">Specifies if the part has a 3rd connector ( such as a Tee )</param>
        ''' </member>

        Private Sub SetFittingPartData(ByVal fitting As DuctFitting, ByVal angle As Double, ByVal partGuid As String, ByVal has3rdConnector As Boolean)
            ' the next bit of code shows how to query the catalog for a part 
            ' First we create a query object, then add the parameters that we are interested in 
            ' to the query. Based on the amount of parameters specified, the results of the query 
            ' may contain multiple records, one record, or no records. 

            Dim width As Double = 12
            Dim height As Double = 12

            Dim query As New DataQuery()
            query.Domain = Domain.DuctComponent
            query.PartGuid = partGuid
            query.AddSizeParameter(Context.ConnectionSizeRectangularHeight, 1, width)
            query.AddSizeParameter(Context.ConnectionSizeRectangularWidth, 1, height)

            query.AddSizeParameter(Context.GeometryPathAngle, 1, angle)

            Dim formatter As New EnumStringFormatter()
            Dim strValue As String = [String].Format(formatter, "{0:CatG}", ConnectionType.Slip)
            query.AddSizeParameter(Context.ConnectionPortPhysicalType, 1, strValue) ' of type Slip 
            query.AddSizeParameter(Context.ConnectionPortPhysicalType, 2, strValue) ' of type Slip

            If has3rdConnector Then
                query.AddSizeParameter(Context.ConnectionPortPhysicalType, 3, strValue) ' of type Slip 
            End If

            ' set the maximum number of records we would like to see 
            Dim maxRecords As Integer = 10

            ' get the ExpandedTable from our query 
            mFittingExpandedTable = PartManager.GetPartTable(query, maxRecords)

            ' get all the records from the table 
            Dim recordCol As DataRecordCollection = mFittingExpandedTable.DataRecords

            mFittingDataRecord = recordCol(0)

            ' this will add the catalog part data to the database object 
            ' 
            PartManager.CreatePartViaRecord(mFittingExpandedTable, mFittingDataRecord, fitting)
        End Sub
#End Region

#Region "AddMvPart"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.AddMvPart </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>This method adds an MvPart to the drawing at the given 
        ''' location.  The particular type of MvPart is specified by the part guid</summary>
        ''' <param name="ptStart">The location for the MvPart</param>
        ''' <param name="partGuid">The guid of the part in the MvPart catalog</param>
        ''' </member>

        Private Function AddMvPart(ByVal ptStart As Point3d, ByVal partGuid As String) As ObjectId
            ' create a new MvPart object 
            ' 
            Dim part As New MultiViewPart()

            ' make sure to call these before setting your geometry on the part 
            part.SetToStandard(mDb)
            part.SetDatabaseDefaults(mDb)
            SetMVPartData(part, partGuid)

            ' now set the layer for the object 
            part.SetDefaultLayer()

            ' finally we will add the duct to the database ands commit the transaction 
            Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
            Dim oid As ObjectId = btr.AppendEntity(part)
            mTransMgr.AddNewlyCreatedDBObject(part, True)
            Return oid
        End Function
#End Region

#Region "SetMVPartData"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.SetMVPartData </name>
        ''' <returntype> void </returntype>
        ''' <summary>This method sets up the MvPart to correspond to the MvPart
        ''' specified by the part guild</summary>
        ''' <param name="part"></param>
        ''' <param name="partGuid"></param>
        ''' </member>

        Private Sub SetMVPartData(ByVal part As MultiViewPart, ByVal partGuid As String)
            ' get first one 
            ' 
            Dim query As New DataQuery()
            query.Domain = Domain.MultiViewPartComponent
            query.PartGuid = partGuid

            ' set the maximum number of records we would like to see 
            Dim maxRecords As Integer = 10

            ' get the ExpandedTable from our query 
            Dim dt As DataExpandedTable = PartManager.GetPartTable(query, maxRecords)

            ' get all the records from the table 
            Dim recordCol As DataRecordCollection = dt.DataRecords

            ' get the first record 
            Dim rec As DataRecord = recordCol(0)

            ' create the mvPart via the record 
            PartManager.CreatePartViaRecord(dt, rec, part)
        End Sub
#End Region

#Region "AngleToRadians"

        ''' <member>
        ''' <name> ConnectionManagerSample.ConnectionManagerSample.AngleToRadians </name>
        ''' <returntype> double </returntype>
        ''' <summary>Convert the given angle in degrees to the corresponding angle in
        ''' radians</summary>
        ''' <param name="value">The angle, in degrees</param>
        ''' </member>

        Public Function AngleToRadians(ByVal value As Double) As Double
            Return value / 180 * System.Math.PI
        End Function
#End Region
    End Class
End Namespace
