#Region "Autodesk Copyright"

' .NET Sample 
' 
' Copyright (c) 2008 by Autodesk, Inc. 
' 
' Permission to use, copy, modify, and distribute this software 
' for any purpose and without fee is hereby granted, provided 
' that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and 
' restricted rights notice below appear in all supporting 
' documentation. 
' 
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
' UNINTERRUPTED OR ERROR FREE. 
' 
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer 
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
' (Rights in Technical Data and Computer Software), as applicable. 
' 

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.Reflection

' AutoCAD MEP Specific 
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.GraphicsSystem
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Elec.DatabaseServices

' create an alias to the AutoCAD Application object 
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports DictionaryProfileDefinition = Autodesk.Aec.Building.DatabaseServices.DictionaryProfileDefinition
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
Imports Device = Autodesk.Aec.Building.Elec.DatabaseServices.Device
Imports MepType = Autodesk.Aec.Building.DatabaseServices.Type

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(DeviceStyles.CreateDeviceStyle))> 

Namespace DeviceStyles

    Public Class CreateDeviceStyle

#Region "Member Data"

        Private mEditor As Editor
        Private mDb As Database
        Private mDoc As Document
        Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager

#End Region

#Region "Setup"

        Protected Sub Setup()
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mDoc = AcadApplication.DocumentManager.MdiActiveDocument
            mTransMgr = mDb.TransactionManager
        End Sub
#End Region

#Region "Command CreateNewDeviceStyle"
        <Autodesk.AutoCAD.Runtime.CommandMethod("CreateNewDeviceStyle", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub CmdCreateNewDeviceStyle()
            Try
                Setup()
                Dim oidStyle As ObjectId = AddDeviceStyle()
                EditPartData(oidStyle)
            Catch ex As System.Exception
                mEditor.WriteMessage(ex.Message)
            End Try
        End Sub
#End Region

#Region "AddDeviceStyle"
        Public Function AddDeviceStyle() As ObjectId
            Using trans As Transaction = mTransMgr.StartTransaction()
                Dim oidReturn As ObjectId
                Dim dict As New DictionaryDeviceStyle(mDb)
                Dim style As New DeviceStyle()

                style.Description = "New Style"
                Dim blockName As String = "mep_block"
                ' makes an assumption the block is in the database 
                Dim bt As BlockTable
                bt = DirectCast(trans.GetObject(mDb.BlockTableId, OpenMode.ForRead), BlockTable)
                If Not bt.Has(blockName) Then
                    AcadApplication.ShowAlertDialog("Block not found")
                    Return ObjectId.Null
                End If

                Dim drm As New DisplayRepresentationManager(mDb)
                Dim devTmp As New Device()
                Dim rc As RXClass = devTmp.GetRXClass()
                Dim ids As ObjectIdCollection = drm.GetAllDisplayRepresentationsWorkForSpecifiedClass(rc)
                Dim dr As DisplayRepresentation = Nothing

                Dim drIdPlan As ObjectId = ObjectId.Null
                Dim drIdModel As ObjectId = ObjectId.Null

                For Each drId As ObjectId In ids
                    dr = TryCast(trans.GetObject(drId, OpenMode.ForRead), DisplayRepresentation)
                    If dr.Name = "AecbDbDispRepDeviceModel" Then
                        drIdModel = dr.Id
                    End If
                    If dr.Name = "AecbDbDispRepDevicePlan" Then
                        drIdPlan = dr.Id
                    End If
                Next

                If drIdModel <> ObjectId.Null Then
                    Dim mvpvd As New MultiViewPartViewDefinition()
                    mvpvd.AllViews = True
                    Dim blkid As ObjectId = bt(blockName)
                    mvpvd.BlockId = blkid
                    mvpvd.Name = "Model"
                    mvpvd.DisplayRepresentationId = drIdModel
                    style.MultiViewPartViewDefinitions.Add(mvpvd)
                End If

                If drIdPlan <> ObjectId.Null Then
                    Dim mvpvd As New MultiViewPartViewDefinition()
                    mvpvd.AllViews = True
                    Dim blkid As ObjectId = bt(blockName)
                    mvpvd.BlockId = blkid
                    mvpvd.Name = "Plan"
                    mvpvd.DisplayRepresentationId = drIdPlan
                    style.MultiViewPartViewDefinitions.Add(mvpvd)
                End If

                Dim mvpc As New MultiViewPartConnector()
                mvpc.Domain = Domain.ElectricalComponent
                mvpc.SystemType = SystemType.PowerAndLightingSystemType
                style.MultiViewPartConnectors.Add(mvpc)

                style.Type = MepType.Lighting
                style.Masking = Masking.Geometry

                Dim classDict As New DictionaryClassificationDefinition(mDb)

                Try
                    Dim classificationId As ObjectId = classDict.GetAt("Device Type")
                    Dim classDef As ClassificationDefinition = DirectCast(trans.GetObject(classificationId, OpenMode.ForRead), ClassificationDefinition)
                    Dim oid As ObjectId = classDef.GetClassification("Lighting")
                    If oid.IsValid Then
                        style.Classifications.Add(oid)
                    End If

                Catch
                End Try

                Try
                    dict.AddNewRecord("NewDeviceStyle", style)
                    mTransMgr.AddNewlyCreatedDBObject(style, True)
                    trans.Commit()
                    oidReturn = dict.GetAt("NewDeviceStyle")
                Catch generatedExceptionName As Autodesk.AutoCAD.Runtime.Exception
                    oidReturn = ObjectId.Null

                Finally
                End Try
                Return oidReturn
            End Using
        End Function

#End Region

#Region "AddDevice"

        Public Function AddDevice(ByVal oid As ObjectId) As ObjectId
            Using trans As Transaction = mTransMgr.StartTransaction()
                Dim oidReturn As ObjectId
                Dim newDevice As New Device()
                newDevice.SetToStandard(mDb)
                newDevice.SetDatabaseDefaults(mDb)
                If Not oid.IsNull Then
                    newDevice.StyleId = oid
                End If
                Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForRead, False), BlockTable)
                Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
                oidReturn = btr.AppendEntity(newDevice)
                mTransMgr.AddNewlyCreatedDBObject(newDevice, True)
                trans.Commit()
                Return oidReturn
            End Using
        End Function

#End Region

#Region "EditPartData"

        Public Sub EditPartData(ByVal oidDevice As ObjectId)
            Try
                Using trans As Transaction = mTransMgr.StartTransaction()
                    Dim style As DeviceStyle = DirectCast(mTransMgr.GetObject(oidDevice, OpenMode.ForWrite, False), DeviceStyle)

                    ' Note: PartManager assumes style is database resident 
                    ' 
                    Dim rec As DataRecord = PartManager.GetPartData(style)

                    Dim index As Integer = 1

                    Dim demandDict As New DictionaryDemandFactorStyle(mDb)

                    ' This is making an assumption that demand factor is there to use 
                    ' 
                    Dim demandFactorStyle As ObjectId = demandDict.GetAt("Lighting - Other")

                    For Each mvpConn As MultiViewPartConnector In style.MultiViewPartConnectors
                        Dim dfLoad As New DataField(Context.ConnectionPropertyLoad, index)
                        dfLoad.ValueDouble = 600
                        rec.DataFields.Add(dfLoad)

                        Dim dfPoles As New DataField(Context.ConnectionPropertyNoOfPoles, index)
                        dfPoles.ValueInteger = 1
                        rec.DataFields.Add(dfPoles)

                        Dim dfVoltage As New DataField(Context.ConnectionPropertyVoltage, index)
                        dfVoltage.ValueDouble = 120
                        rec.DataFields.Add(dfVoltage)

                        Dim dfDemand As New DataField(Context.ConnectionPropertyDemandFactor, index)
                        dfDemand.ValueObjectId = demandFactorStyle
                        rec.DataFields.Add(dfDemand)

                        index += 1
                    Next

                    ' Now add layer key override. This is a singleton df, so index should be zero.
                    ' 
                    Dim dfLayerKey As New DataField(Context.LayerKey, 0)
                    dfLayerKey.ValueString = "E-DV-DEVICE"
                    rec.DataFields.Add(dfLayerKey)

                    PartManager.SetPartData(style, rec)

                    trans.Commit()
                End Using
            Catch ex As System.Exception
                mEditor.WriteMessage(ex.Message)
            End Try
        End Sub
#End Region
    End Class
End Namespace