#Region "Autodesk Copyright "
'
'  .NET Sample
'
'  Copyright (c) 2007 by Autodesk, Inc.
'
'  Permission to use, copy, modify, and distribute this software
'  for any purpose and without fee is hereby granted, provided
'  that the above copyright notice appears in all copies and
'  that both that copyright notice and the limited warranty and
'  restricted rights notice below appear in all supporting
'  documentation.
'
'  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'  AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'  UNINTERRUPTED OR ERROR FREE.
'
'  Use, duplication, or disclosure by the U.S. Government is subject to
'  restrictions set forth in FAR 52.227-19 (Commercial Computer
'  Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'  (Rights in Technical Data and Computer Software), as applicable.
'
#End Region
#Region "References"
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices
Imports Autodesk.Aec.Building.Elec.DatabaseServices
Imports Autodesk.Aec.Building.Piping.DatabaseServices
Imports Autodesk.Aec.Building.Plumbing.DatabaseServices
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(Nothing)> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(Nothing)> 

Namespace MepUtilities
    Public Class PromptingUtilites
#Region "PromptForObject"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForObject </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>This method prompts the user to select an object.  The caller can supply
        ''' the prompt string,  the type of object to be selected,  and the message to be
        ''' presented if the user tries to pick the wrong type of object.</summary>
        ''' <param name="prompt">The message used to ask the user to pick an object</param>
        ''' <param name="rejectMessage">The message used to tell the user they picked the wrong type of object</param>
        ''' <param name="objType">The type of object to be selected</param>
        ''' <param name="exactMatch"></param>
        ''' </member>

        Public Shared Function PromptForObject(ByVal prompt As String, ByVal rejectMessage As String, ByVal objType As System.Type, ByVal exactMatch As Boolean) As ObjectId
            Dim prOptions As PromptEntityOptions = New PromptEntityOptions("")

            ' Set up the prompt options
            If prompt Is Nothing OrElse prompt.Length = 0 Then
                prOptions.Message = "" & Microsoft.VisualBasic.Chr(10) & "Select object: "
            Else
                prOptions.Message = prompt
            End If

            If rejectMessage Is Nothing OrElse rejectMessage.Length = 0 Then
                prOptions.SetRejectMessage("" & Microsoft.VisualBasic.Chr(10) & "Object must be of type " + objType.ToString)
            Else
                prOptions.SetRejectMessage(rejectMessage)
            End If

            prOptions.AddAllowedClass(objType, exactMatch)

            Try
                ' prompt the user to select a member
                '
                Dim prResult As PromptEntityResult = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor.GetEntity(prOptions)

                If prResult.Status = PromptStatus.Cancel Then
                    Return Autodesk.AutoCAD.DatabaseServices.ObjectId.Null
                End If

                ' get the ObjectId from the selected member 
                '
                Return prResult.ObjectId
            Catch ex As SystemException
                Throw ex
            End Try
        End Function

#End Region

#Region "PromptForDuct"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForDuct </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a duct</summary>
        ''' <returns>The object id of the duct</returns>
        ''' <param name="prompt">The string asking the user to select the duct</param>
        ''' </member>

        Public Shared Function PromptForDuct(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select duct: "
                End If

                Return PromptForObject(prompt, "", GetType(Duct), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForFlexDuct"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForFlexDuct </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a flex duct</summary>
        ''' <returns>The object id of the flex duct</returns>
        ''' <param name="prompt">The string asking the user to select the flex duct</param>
        ''' </member>

        Public Shared Function PromptForFlexDuct(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select flex duct: "
                End If

                Return PromptForObject(prompt, "", GetType(DuctFlex), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForDuctFitting"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForDuctFitting </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a duct fitting</summary>
        ''' <returns>The object id of the duct fitting</returns>
        ''' <param name="prompt">The string asking the user to select the duct fitting</param>
        ''' </member>

        Public Shared Function PromptForDuctFitting(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select duct fitting: "
                End If

                Return PromptForObject(prompt, "", GetType(DuctFitting), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForPipe"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForPipe </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a pipe</summary>
        ''' <returns>The object id of the pipe</returns>
        ''' <param name="prompt">The string asking the user to select the pipe</param>
        ''' </member>

        Public Shared Function PromptForPipe(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select pipe: "
                End If

                Return PromptForObject(prompt, "", GetType(Pipe), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForPipeFlex"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForPipeFlex </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a flex pipe</summary>
        ''' <returns>The object id of the flex pipe</returns>
        ''' <param name="prompt">The string asking the user to select the flex pipe</param>
        ''' </member>

        Public Shared Function PromptForPipeFlex(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select flex pipe: "
                End If

                Return PromptForObject(prompt, "", GetType(PipeFlex), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForPipeFitting"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForPipeFitting </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a pipe fitting</summary>
        ''' <returns>The object id of the pipe fitting</returns>
        ''' <param name="prompt">The string asking the user to select the pipe fitting</param>
        ''' </member>

        Public Shared Function PromptForPipeFitting(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select pipe fitting: "
                End If

                Return PromptForObject(prompt, "", GetType(PipeFlex), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForDevice"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForDevice </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a device</summary>
        ''' <returns>The object id of the device</returns>
        ''' <param name="prompt">The string asking the user to select the device</param>
        ''' </member>

        Public Shared Function PromptForDevice(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select device: "
                End If

                Return PromptForObject(prompt, "", GetType(Device), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function
#End Region

#Region "PromptForWire"

        ''' <member>
        ''' <name> MepUtilities.PromptingUtilites.PromptForWire </name>
        ''' <returntype> Autodesk.AutoCAD.DatabaseServices.ObjectId </returntype>
        ''' <summary>Prompt the user to select a wire</summary>
        ''' <returns>The object id of the wire</returns>
        ''' <param name="prompt">The string asking the user to select the wire</param>
        ''' </member>

        Public Shared Function PromptForWire(ByVal prompt As String) As ObjectId
            Try
                If prompt.Length = 0 Then
                    prompt = "" & Microsoft.VisualBasic.Chr(10) & "Select wire: "
                End If

                Return PromptForObject(prompt, "", GetType(Wire), True)
            Catch ex As SystemException
                Throw ex
            End Try
        End Function

#End Region
    End Class
End Namespace

