#Region "Autodesk Copyright"

' .NET Sample 
' 
' Copyright (c) 2008 by Autodesk, Inc. 
' 
' Permission to use, copy, modify, and distribute this software 
' for any purpose and without fee is hereby granted, provided 
' that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and 
' restricted rights notice below appear in all supporting 
' documentation. 
' 
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
' UNINTERRUPTED OR ERROR FREE. 
' 
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer 
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
' (Rights in Technical Data and Computer Software), as applicable. 
' 

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.IO
Imports System.Collections
Imports System.Reflection

' AutoCAD MEP Specific 
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices

Imports Autodesk.Aec.Building.Elec
Imports Autodesk.Aec.Building.Elec.DatabaseServices
Imports ElecApp = Autodesk.Aec.Building.Elec.ApplicationServices

Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
Imports AecUtility = Autodesk.Aec.ApplicationServices.Utility

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(PanelPropertiesSample.PanelPropertiesSample))> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(Nothing)> 
Namespace PanelPropertiesSample
    Public Class PanelPropertiesSample
#Region "static memmbers"

        Shared deviceIds As ObjectIdCollection
        Shared singlePoleCircuitIds As ObjectIdCollection
        Shared calCircuits As CalculateAllCircuits

#End Region

#Region "AddPanels"
        <Autodesk.AutoCAD.Runtime.CommandMethod("AddPanels", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub CmdAddPanels()
            Dim projectDatabase As Database
            Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
            deviceIds = New ObjectIdCollection()

            Dim epdName As String = "ProjectDatabase.epd"
            Dim dwgPath As String = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), epdName)
            projectDatabase = CreateDwg(dwgPath)
            projectDatabase.SaveAs(epdName, DwgVersion.Current)
            SetElectricalProjectDatabase(projectDatabase, "")
            projectDatabase.Dispose()

            SetElectricalProjectDatabase(db, dwgPath)

            singlePoleCircuitIds = New ObjectIdCollection()
            CreatePanel(db, "Main", New Point3d(0, 0, 0))
            singlePoleCircuitIds = New ObjectIdCollection()
            CreatePanel(db, "Sub Panel - One", New Point3d(0, 120, 0))

            CreateDevice(db, New Point3d(0, 12, 0))
            CreateDevice(db, New Point3d(60, 12, 0))
            CreateDevice(db, New Point3d(120, 12, 0))
            CreateDevice(db, New Point3d(180, 12, 0))
            CircuitDevices(db)
        End Sub
#End Region

#Region "GetPanelProperties"
        <Autodesk.AutoCAD.Runtime.CommandMethod("GetPanelProperties", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub GetPanelProperties()
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Try
                Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
                Using trans As Transaction = db.TransactionManager.StartTransaction()
                    ' create a new circuit calulation helper object 
                    ' 
                    CircuitServices.SyncProjectDatabase(db, "ProjectDatabase.epd")

                    calCircuits = New CalculateAllCircuits(True, db)

                    ' get an array of all the panels from the helper, this includes external panels, if any 
                    ' 
                    Dim arrPanels As AbstractPanel() = calCircuits.GetAllPanels()

                    Dim panelName As String
                    Dim panelDwgName As String
                    Dim panelFedFrom As String = ""
                    Dim external As Boolean = False
                    For i As Integer = 0 To arrPanels.Length - 1

                        ' now lets loop through all the panels 
                        ' 
                        Dim abPanel As AbstractPanel = arrPanels(i)
                        Dim panelProps As Autodesk.Aec.Building.Elec.PanelProperties
                        panelProps = calCircuits.GetPanelProperties(abPanel)

                        If Not arrPanels(i).PanelId.IsNull Then
                            ' panel is local to the current drawing, so let open it 
                            ' open the local panel for read 
                            ' 
                            Dim panel As Panel = DirectCast(trans.GetObject(arrPanels(i).PanelId, OpenMode.ForRead), Panel)

                            ' extract some properties from the panel 
                            ' 
                            panelName = panel.Name
                            panelDwgName = panel.Database.Filename
                            panelFedFrom = panel.FedFrom.ToString()
                            external = False
                        Else
                            panelName = abPanel.PanelName
                            panelDwgName = abPanel.PanelBelongDrawing
                            external = abPanel.IsExternalPanel
                        End If

                        ed.WriteMessage(vbNewLine + "Name: " + panelName)
                        ed.WriteMessage(vbNewLine + vbTab + "Drawing belongs to: " + panelDwgName)
                        ed.WriteMessage(vbNewLine + vbTab + "Is external: " + external.ToString())
                        PrintPanelProperties(panelProps)
                        PrintCircuitInfo(abPanel)

                        ed.WriteMessage(vbNewLine)
                    Next
                    trans.Commit()
                End Using
            Catch ex As System.Exception
                ed.WriteMessage(ex.Message)
            End Try
        End Sub
#End Region

#Region "CircuitDevices"
        Public Shared Sub CircuitDevices(ByVal db As Database)
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                For indx As Integer = 0 To deviceIds.Count - 1
                    Dim device As Device = DirectCast(trans.GetObject(deviceIds(indx), OpenMode.ForWrite), Device)
                    Dim instanceData As DataRecord = PartManager.GetEngineeringData(device)
                    Dim circuitDataField As New DataField(Context.ConnectionPropertyCircuit, 1)
                    circuitDataField.ValueObjectId = singlePoleCircuitIds(indx)
                    instanceData.DataFields.Add(circuitDataField)
                    PartManager.SetEngineeringData(device, instanceData)
                    device.Dispose()
                Next
                trans.Commit()
            End Using
        End Sub
#End Region

#Region "CreateDevice"
        Public Shared Sub CreateDevice(ByVal db As Database, ByVal location As Point3d)
            ' set the working database to the one passes in so PartManager can 
            ' work on the correct database 
            ' 
            Dim curDb As Database = HostApplicationServices.WorkingDatabase
            HostApplicationServices.WorkingDatabase = db

            Dim deviceId As ObjectId = ObjectId.Null
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim newDevice As New Device()
                newDevice.SetToStandard(db)
                newDevice.SetDatabaseDefaults(db)
                newDevice = DirectCast(PartManager.CreatePartEntityViaStyle("Receptacles (US Imperial)", "Duplex Receptacle", GetType(Device)), Device)
                deviceId = newDevice.ObjectId
                Dim bt As BlockTable = DirectCast(trans.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
                Dim btr As BlockTableRecord = DirectCast(trans.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
                newDevice.Location = location
                newDevice.Elevation = 18
                deviceIds.Add(newDevice.ObjectId)
                newDevice.Dispose()
                trans.Commit()
            End Using

            ' now set the property load on the device 
            ' 
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim device As Device = DirectCast(trans.GetObject(deviceId, OpenMode.ForWrite), Device)
                Dim dataRecord As DataRecord = PartManager.GetEngineeringData(device)
                Dim dataField As New DataField(Context.ConnectionPropertyLoad, 1)
                dataField.ValueDouble = 180
                dataRecord.DataFields.Add(dataField)
                PartManager.SetEngineeringData(device, dataRecord)
                device.Dispose()
                trans.Commit()
            End Using

            ' reset the working database 
            ' 
            HostApplicationServices.WorkingDatabase = curDb
        End Sub
#End Region

#Region "CreateDwg"
        Public Shared Function CreateDwg(ByVal fileName As String) As Database
            Dim db As New Database(True, False)
            db.SaveAs(fileName, DwgVersion.Current)
            Return db
        End Function
#End Region

#Region "CreatePanel"
        Public Shared Function CreatePanel(ByVal db As Database, ByVal panelName As String, ByVal location As Point3d) As ObjectId
            Dim panelId As ObjectId = ObjectId.Null
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim panel As New Panel()
                panel.SetToStandard(db)
                panel.SetDatabaseDefaults(db)

                ' load style 
                LoadPanelStyles(db)

                ' set panel properties 
                panel.StyleId = GetPanelStyleId("Recessed 1", db)
                panel.Name = panelName
                panel.Location = location
                panel.MainType = PanelMainType.MainCircuitBreaker
                panel.MainSize = 225
                panel.PhaseToNeutralVoltage = 120
                panel.PhaseToPhaseVoltage = 208
                panel.PhaseType = PanelPhaseType.Panel3Phase
                panel.Mounting = PanelMounting.Surface

                Dim bt As BlockTable = DirectCast(trans.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
                Dim btr As BlockTableRecord = DirectCast(trans.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
                panelId = btr.AppendEntity(panel)
                trans.AddNewlyCreatedDBObject(panel, True)
                trans.Commit()
                panel.Dispose()
            End Using

            CreateCircuits(panelId, db)
            Return panelId
        End Function
#End Region

#Region "GetDeviceDirectory"
        Public Shared Function GetDeviceDirectory() As String
            Dim dir As String = ElecApp.ElecDBVariables.StringDrawingDirectoryDevice
            Return dir
        End Function
#End Region

#Region "SetElectricalProjectDatabase"
        Public Shared Sub SetElectricalProjectDatabase(ByVal db As Database, ByVal epdName As String)
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim oid As ObjectId = ElecApp.ElecDBVariables.GetInstance(db, True)
                Dim dbv As ElecApp.ElecDBVariables = DirectCast(trans.GetObject(oid, OpenMode.ForWrite), ElecApp.ElecDBVariables)
                dbv.ElectricalProjectDatabase = epdName
                Dim cir As New Autodesk.Aec.Building.Elec.Circuit()
                CircuitServices.SyncProjectDatabase(db, epdName)
                trans.Commit()
            End Using
        End Sub
#End Region

#Region "GetPanelDirectory"
        Public Shared Function GetPanelDirectory() As String
            Dim dir As String = ElecApp.ElecDBVariables.StringDrawingDirectoryPanel
            Return dir
        End Function
#End Region

#Region "LoadPanelStyles"
        Public Shared Sub LoadPanelStyles(ByVal destDb As Database)
            Dim fileName As String = "Panels (US Imperial).dwg"
            Dim panelStylesFile As String = Path.Combine(GetPanelDirectory(), fileName)
            Dim db As New Database(False, False)
            db.ReadDwgFile(panelStylesFile, FileShare.ReadWrite, False, Nothing)
            ' here we use the Clone helper from ACA to clone all the panel style into 
            ' the destDB drawing 
            Dim ch As New AecUtility.CloningHelper()
            Dim panelStyles As New DictionaryPanelStyle(db)
            ch.MergeType = AecUtility.DictionaryRecordMergeBehavior.Normal
            ch.Clone(db, destDb, panelStyles.Records, panelStyles.RecordType, True)
            ch.Dispose()
            db.Dispose()
        End Sub
#End Region

#Region "LoadDeviceStyle"
        Public Shared Function LoadDeviceStyle(ByVal destDb As Database, ByVal libFileName As String, ByVal deviceStyleName As String) As ObjectId
            Dim deviceStyleId As ObjectId = ObjectId.Null
            Dim deviceStylesFile As String = Path.Combine(GetDeviceDirectory(), libFileName)
            Dim db As New Database(False, False)
            db.ReadDwgFile(deviceStylesFile, FileShare.ReadWrite, False, Nothing)
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim dds As New DictionaryDeviceStyle(db)
                If dds.Has(deviceStyleName, trans) Then
                    ' here we use the Clone helper from ACA to clone all the panel style into 
                    ' the destDB drawing 
                    ' 
                    Dim ch As New AecUtility.CloningHelper()
                    Dim sourceDeviceStyleId As ObjectId = dds.GetAt(deviceStyleName)
                    Dim styleCollection As New ObjectIdCollection()
                    styleCollection.Add(sourceDeviceStyleId)
                    ch.MergeType = AecUtility.DictionaryRecordMergeBehavior.Normal
                    ch.Clone(db, destDb, styleCollection, dds.RecordType, True)
                End If
                trans.Commit()
            End Using

            db.Dispose()

            ' now get the device style object id that we were looking for 
            ' 
            Using trans As Transaction = destDb.TransactionManager.StartTransaction()
                Dim dds As New DictionaryDeviceStyle(destDb)
                If dds.Has(deviceStyleName, trans) Then
                    deviceStyleId = dds.GetAt(deviceStyleName)
                End If
                trans.Commit()
            End Using

            Return deviceStyleId
        End Function
#End Region

#Region "GetPanelStyleId"
        Public Shared Function GetPanelStyleId(ByVal name As String, ByVal db As Database) As ObjectId
            Dim panelStyleId As ObjectId = ObjectId.Null
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                Dim panelStyles As New DictionaryPanelStyle(db)
                If panelStyles.Has(name, trans) Then
                    panelStyleId = panelStyles.GetAt(name)
                End If
                trans.Commit()
            End Using
            Return panelStyleId
        End Function
#End Region

#Region "CreateCircuits"
        Public Shared Sub CreateCircuits(ByVal oidPanel As ObjectId, ByVal db As Database)
            ' set the working database to the one passed in 
            ' 
            Dim curDb As Database = HostApplicationServices.WorkingDatabase
            HostApplicationServices.WorkingDatabase = db

            Dim circuitProperties As New ArrayList()

            ' add new circuit properties to the array 
            circuitProperties.Add(New CircuitProperty(Context.CircuitRating, 20))
            circuitProperties.Add(New CircuitProperty(Context.ConnectionPropertyVoltage, 208))
            circuitProperties.Add(New CircuitProperty(Context.ConnectionPropertyNoOfPoles, 3))

            ' now add new circuits 
            CircuitServices.NewCircuit("1,3,5", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties)
            CircuitServices.NewCircuit("2,4,6", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties)

            ' clear the array 
            circuitProperties.Clear()

            ' again add new circuit propeties to the array 
            circuitProperties.Add(New CircuitProperty(Context.CircuitRating, 20))
            circuitProperties.Add(New CircuitProperty(Context.ConnectionPropertyVoltage, 120))
            circuitProperties.Add(New CircuitProperty(Context.ConnectionPropertyNoOfPoles, 1))

            ' now create new circuits and add the id to the singlePoleCircuitId array. 
            singlePoleCircuitIds.Add(CircuitServices.NewCircuit("7", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties))
            singlePoleCircuitIds.Add(CircuitServices.NewCircuit("8", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties))
            singlePoleCircuitIds.Add(CircuitServices.NewCircuit("9", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties))
            singlePoleCircuitIds.Add(CircuitServices.NewCircuit("10", "", oidPanel, ObjectId.Null, SystemType.PowerAndLightingSystemType, circuitProperties))

            ' reset the working database to the orginal value 
            ' 
            HostApplicationServices.WorkingDatabase = curDb
        End Sub
#End Region

#Region "PrintPanelProperties"
        Private Sub PrintPanelProperties(ByVal panelProps As Autodesk.Aec.Building.Elec.PanelProperties)
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' using reflection to displat the properties of the panel 
            ' 
            Dim type As System.Type = GetType(Autodesk.Aec.Building.Elec.PanelProperties)
            Dim propInfos As PropertyInfo() = type.GetProperties()
            For Each propInfo As PropertyInfo In propInfos
                If propInfo.DeclaringType.FullName.CompareTo(type.FullName) = 0 Then
                    Dim objValue As Object = propInfo.GetValue(panelProps, Nothing)
                    ed.WriteMessage(vbNewLine & vbTab + propInfo.Name + ": " + objValue.ToString())
                End If
            Next
        End Sub
#End Region

#Region "PrintCircuitInfo"
        Private Sub PrintCircuitInfo(ByVal abPanel As AbstractPanel)
            Dim db As Database = HostApplicationServices.WorkingDatabase
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim panelid As ObjectId = abPanel.PanelId

            Dim oidcollection As ObjectIdCollection = CircuitServices.GetAllCircuitsByPanelAndSystemType(panelid, SystemType.PowerAndLightingSystemType)
            Using trans As Transaction = db.TransactionManager.StartTransaction()
                For Each oid As ObjectId In oidcollection
                    ' open the circuit 
                    ' 
                    Dim cir As Autodesk.Aec.Building.Elec.DatabaseServices.Circuit = DirectCast(trans.GetObject(oid, OpenMode.ForRead), Autodesk.Aec.Building.Elec.DatabaseServices.Circuit)
                    Dim devices As ObjectIdCollection = CircuitServices.GetDevicesInCircuit(oid, oid.Database)
                    ed.WriteMessage(vbNewLine & vbTab & vbTab & "Circuit Name: " + cir.AlternateName)
                    If devices.Count > 0 Then
                        ed.WriteMessage(vbNewLine & vbTab & vbTab & "Circuit contains the following devices:")
                        For Each deviceId As ObjectId In devices
                            Dim device As Device = DirectCast(trans.GetObject(deviceId, OpenMode.ForRead), Device)
                            Dim deviceStyle As DeviceStyle = DirectCast(trans.GetObject(device.StyleId, OpenMode.ForRead), DeviceStyle)
                            ed.WriteMessage(vbNewLine & vbTab & vbTab & vbTab & "Device name: " + deviceStyle.Name)
                        Next
                    Else
                        ed.WriteMessage(vbNewLine & vbTab & vbTab & "Circuit contains no devices.")
                    End If
                Next
            End Using
        End Sub
#End Region
    End Class
End Namespace