#Region " Autodesk Copyright "

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region " Namespaces References "

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics


' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.GeomeTry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices
Imports Autodesk.Aec.Building.Elec.DatabaseServices
Imports Autodesk.Aec.Building.Plumbing.DatabaseServices
Imports Autodesk.Aec.Building.Piping.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(SystemDefs.SystemDefinitionsExample))> 

Namespace SystemDefs
    Public Class SystemDefinitionsExample

#Region "GetDuctSysDefs"
        ' Define a new command GetDuctSysDefs
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("SystemDefs", "GetDuctSysDefs", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub getDuctSysDefs()
            Try
                ' Get the active database
                '
                Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
                Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
                Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

                ' Start transation
                '
                Dim myT As Transaction
                myT = transMgr.StartTransaction()

                ' Get the duct system defs dictionary for this database                   
                '
                Dim ddsd As DictionaryDuctSystemDefinition = New DictionaryDuctSystemDefinition(db)

                ' Now loop through each entry in the dictionary and display various properties
                '
                Dim oid As ObjectId
                For Each oid In ddsd.Records
                    ' Open the duct system definition for read
                    '
                    Dim dsd As DuctSystemDefinition = CType(transMgr.GetObject(oid, OpenMode.ForRead, False), DuctSystemDefinition)

                    editor.WriteMessage(vbNewLine + "System Name: " + dsd.Name)
                    editor.WriteMessage(vbNewLine + "-------------------------------------------------------------------")
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Description: " + dsd.Description)
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Domain: " + dsd.Domain.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Enable 1 Line Display: " + dsd.Enable1LineDisplay.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Enable Graphic 1 LineDisplay: " + dsd.EnableGraphic1LineDisplay.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Exclude From ShrinkWrap: " + dsd.ExcludeFromShrinkWrap.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "LayerKey: " + dsd.LayerKey)
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "LabelName: " + dsd.LabelName)
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Family Name: " + dsd.FamilyName)
                    Dim vel As String = Context.Velocity.ToString()
                    Dim epVel As EngineeringParameter = dsd.GetEngineeringParameter(vel)
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Velocity: " + epVel.Value.ToString() + epVel.Units)
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Friction: " + dsd.GetEngineeringParameter("Friction").Value.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Roughness: " + dsd.GetEngineeringParameter("Roughness").Value.ToString())
                    editor.WriteMessage(vbNewLine + vbTab + vbTab + "Density: " + dsd.GetEngineeringParameter("Density").Value.ToString())
                    editor.WriteMessage(vbNewLine + "")

                Next

                ' make sure to commit the transaction
                '
                myT.Commit()

                ' Catch any errors found
                '
            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
                Debug.WriteLine(be.Message)
            Catch ex As System.Exception
                Debug.Write(String.Format(vbNewLine + "Caught the following Exception:{0}", ex.ToString()))
            End Try
        End Sub
#End Region

#Region "ListSysDefs"
        ' Define a new command ListSysDefs
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("SystemDefs", "ListSysDefs", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub ListSystemDefinitions()

            Try
                ' Get the active database
                '
                Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
                Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database

                ' Get the system dictionaries for this database
                '
                Dim ddsd As DictionaryDuctSystemDefinition = New DictionaryDuctSystemDefinition(db)
                Dim desd As DictionaryElectricalSystemDefinition = New DictionaryElectricalSystemDefinition(db)
                Dim dpsd As DictionaryPipeSystemDefinition = New DictionaryPipeSystemDefinition(db)
                Dim dplsd As DictionaryPlumbingSystemDefinition = New DictionaryPlumbingSystemDefinition(db)

                ' Print out all systems for each domain
                '
                editor.WriteMessage(vbNewLine + "-------------------------------Duct--------------------------------")
                ListSystemDefinition(ddsd)
                editor.WriteMessage(vbNewLine + "----------------------------Electrical-----------------------------")
                ListSystemDefinition(desd)
                editor.WriteMessage(vbNewLine + "------------------------------Piping-------------------------------")
                ListSystemDefinition(dpsd)
                editor.WriteMessage(vbNewLine + "-----------------------------Plumbing------------------------------")
                ListSystemDefinition(dplsd)

                ' Catch any errors found
                '
            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException

                Debug.WriteLine(be.Message)

            Catch ex As System.Exception

                Debug.Write(String.Format(vbNewLine + "Caught the following Exception:{0}", ex.ToString()))

            End Try

        End Sub
#End Region

#Region "ListSystemDefinition"

        Public Sub ListSystemDefinition(ByVal sysDef As Autodesk.Aec.DatabaseServices.Dictionary)
            ' This function will iterate all systems in a system collection
            ' and print its name and other information
            Try
                ' Get the active database to start transaction
                '
                Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
                Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = sysDef.Database.TransactionManager

                Dim myT As Transaction
                myT = transMgr.StartTransaction()

                ' Now loop through each entry in the dictionary and display various properties
                '
                Dim oid As ObjectId
                For Each oid In sysDef.Records
                    ' Open the system definition for read
                    '
                    Dim dsd As SystemDefinition = CType(transMgr.GetObject(oid, OpenMode.ForRead, False), SystemDefinition)

                    Try
                        ' The standard system will be ignored in this example
                        '
                        If dsd.Name <> "Standard" Then
                            editor.WriteMessage(vbNewLine + "       System Name: " + dsd.Name)
                            editor.WriteMessage(vbNewLine + "    Alternate Name: " + dsd.AlternateName)
                            editor.WriteMessage(vbNewLine + "       Description: " + dsd.Description)
                            editor.WriteMessage(vbNewLine + "       Family Name: " + dsd.FamilyName)
                            editor.WriteMessage(vbNewLine + "         LabelName: " + dsd.LabelName)
                            editor.WriteMessage(vbNewLine + "          LayerKey: " + dsd.LayerKey + vbNewLine)
                        End If

                        ' Catch any errors found
                        '
                    Catch acadEx As Autodesk.AutoCAD.Runtime.Exception
                        If acadEx.ErrorStatus <> ErrorStatus.KeyNotFound Then
                            Debug.Assert(False)
                        End If
                        editor.WriteMessage(vbNewLine)
                    End Try
                Next

                ' make sure to commit the transaction
                '
                myT.Commit()

                ' Catch any errors found
                '
            Catch be As Autodesk.Aec.Building.DatabaseServices.BuildingException
                Debug.WriteLine(be.Message)
            Catch ex As System.Exception
                Debug.Write(String.Format(vbNewLine + "Caught the following Exception:{0}", ex.ToString()))
            End Try
        End Sub

#End Region

#Region "CreateDuctSysDef"
        ' Define a new command CreateDuctSysDef
        '
        <Autodesk.AutoCAD.Runtime.CommandMethod("SystemDefs", "CreateDuctSysDef", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub CreateDuctSysDef()
            ' This command will create and add a duct system to the collection of duct systems
            '

            ' Get the current database to start transaction
            '
            Dim editor As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            Dim db As Database = AcadApplication.DocumentManager.MdiActiveDocument.Database
            Dim transMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager

            ' Start transaction
            '
            Using myT As Transaction = transMgr.StartTransaction()

                ' Get access to the collection of duct systems
                '
                Dim ductSystemDefDic As New DictionaryDuctSystemDefinition(db)

                ' Create a new duct system
                '
                Dim newDsd As New DuctSystemDefinition()

                ' Call setToStandard for the new duct system to setup default values
                '
                newDsd.SetToStandard(db)

                ' Set data on the system
                '
                newDsd.Description = "Created by MEP .NET API"
                newDsd.Enable1LineDisplay = True
                newDsd.EnableGraphic1LineDisplay = True
                newDsd.ExcludeFromShrinkWrap = False
                newDsd.LayerKey = "DUCTSYSTEM"
                newDsd.LabelName = "NAS"
                newDsd.FamilyName = "DUCT"
                newDsd.InitializeDefaultEngineeringData(db)

                ' Add the new system to the collection of systems
                '
                ductSystemDefDic.AddNewRecord(".Net API Sample", newDsd)

                ' Add the new system to the database
                '
                myT.AddNewlyCreatedDBObject(newDsd, True)

                ' make sure to commit the transaction
                '
                myT.Commit()

            End Using

        End Sub

#End Region

    End Class
End Namespace

